@@ -14,6 +14,7 @@
 **************************************************************************/
 
 #include <stdio.h>
+#include <limits.h>
 #include "version.h"
 #include "compile.h"
 #include "shhopt.h"
@@ -26,23 +27,71 @@
 /* The following are set by pm_init(), then used by subsequent calls to other
    pm_xxx() functions.
    */
-static char* progname;
-static bool pm_show_version;
-    /* Programs should show version information (per the user's
-       --version option) 
-    */
-static bool showmessages;  
+static const char* pm_progname;
+static bool pm_showmessages;  
     /* Programs should display informational messages (because the user didn't
        specify the --quiet option).
     */
 
+void
+pm_usage( const char usage[] ) {
+    fprintf( stderr, "usage:  %s %s\n", pm_progname, usage );
+    exit( 1 );
+    }
+
+void
+pm_perror(const char reason[] ) {
+    const char* e;
+
+#ifdef A_STRERROR
+    e = strerror(errno);
+#else /* A_STRERROR */
+    e = sys_errlist[errno];
+#endif /* A_STRERROR */
+
+    if ( reason != 0 && reason[0] != '\0' )
+        pm_error( "%s - %s", reason, e );
+    else
+        pm_error( "%s", e );
+    }
+
+void GNU_PRINTF_ATTR
+pm_message(const char format[], ... ) {
+
+    va_list args;
+
+    va_start( args, format );
+
+    if ( pm_showmessages )
+        {
+        fprintf( stderr, "%s: ", pm_progname );
+        (void) vfprintf( stderr, format, args );
+        fputc( '\n', stderr );
+        }
+    va_end( args );
+    }
+
+void GNU_PRINTF_ATTR
+pm_error(const char format[], ... ) {
+    va_list args;
+
+    va_start( args, format );
+
+    fprintf( stderr, "%s: ", pm_progname );
+    (void) vfprintf( stderr, format, args );
+    fputc( '\n', stderr );
+    va_end( args );
+    exit( 1 );
+    }
+
+
 /* Variable-sized arrays. */
 
 char*
 pm_allocrow(int const cols, int const size) {
     register char* itrow;
 
-    itrow = (char*) malloc( cols * size );
+    itrow = (char*) malloc2( cols , size );
     if ( itrow == (char*) 0 )
         pm_error( "out of memory allocating a row" );
     return itrow;
@@ -60,10 +109,10 @@ pm_allocarray(int const cols, int const rows, int const size )  {
     char** its;
     int i;
 
-    its = (char**) malloc( rows * sizeof(char*) );
+    its = (char**) malloc2( rows, sizeof(char*) );
     if ( its == (char**) 0 )
         pm_error( "out of memory allocating an array" );
-    its[0] = (char*) malloc( rows * cols * size );
+    its[0] = (char*) malloc3( rows, cols, size );
     if ( its[0] == (char*) 0 )
         pm_error( "out of memory allocating an array" );
     for ( i = 1; i < rows; ++i )
@@ -81,10 +130,12 @@ char**
 pm_allocarray(int const cols, int const rows, int const size) {
     char** its;
     int i;
-    its = (char**) malloc( (rows + 1) * sizeof(char*) );
+
+    overflow_add(rows, 1);
+    its = (char**) malloc2( (rows + 1),  sizeof(char*) );
     if ( its == (char**) 0 )
         pm_error( "out of memory allocating an array" );
-    its[rows] = its[0] = (char*) malloc( rows * cols * size );
+    its[rows] = its[0] = (char*) malloc3( rows. cols, size );
     if ( its[0] != (char*) 0 )
         for ( i = 1; i < rows; ++i )
             its[i] = &(its[0][i * cols * size]);
@@ -139,35 +190,6 @@ pm_keymatch(char * const strarg, char * const keywordarg, int const minchars) {
 }
 
 
-/* Wrapper for Shhopt, to get it into the shared library */
-
-void 
-pm_optParseOptions(int * const argc, char * argv[],
-                   optStruct opt[], int const allowNegNum) {
-
-    optParseOptions(argc, argv, opt, allowNegNum);
-}
-
-
-
-void 
-pm_optParseOptions2(int * const argc_p, char * argv[],
-                    const optStruct2 opt, const unsigned long flags) {
-
-    optParseOptions2(argc_p, argv, opt, flags);
-}
-
-
-void
-pm_optParseOptions3(int * const argc_p, char *argv[], const optStruct3 opt, 
-                    const unsigned int optStructSize, 
-                    const unsigned long flags) {
-
-    optParseOptions3(argc_p, argv, opt, optStructSize, flags);
-
-}
-
-
 /* Log base two hacks. */
 
 int
@@ -249,112 +271,97 @@ pm_lcm(const unsigned int x,
 /* Initialization. */
 
 
-void
-pm_init(int * const argcP, char * argv[]) {
-
-    int argn, i;
-    char *rgbdef;
-
-#ifndef VMS
-    /* Extract program name. */
-    progname = strrchr( argv[0], '/');
-#else
-{   char **temp_argv = argv;
+#ifdef VMS
+static const char *
+vmsProgname(int * const argcP, char * argv[]) {   
+    char **temp_argv = argv;
     int old_argc = *argcP;
     int i;
+    const char * retval;
+    
     getredirection( argcP, &temp_argv );
     if (*argcP > old_argc) {
         /* Number of command line arguments has increased */
-        fprintf( stderr, "Sorry!! getredirection() for VMS has changed the argument list!!!\n");
-        fprintf( stderr, "This is intolerable at the present time, so we must stop!!!\n");
+        fprintf( stderr, "Sorry!! getredirection() for VMS has "
+                 "changed the argument list!!!\n");
+        fprintf( stderr, "This is intolerable at the present time, "
+                 "so we must stop!!!\n");
         exit(1);
     }
     for (i=0; i<*argcP; i++)
         argv[i] = temp_argv[i];
-    }
-    if ( progname == NULL ) progname = rindex( argv[0], ']');
-    if ( progname == NULL ) progname = rindex( argv[0], '>');
+    retval = strrchr( argv[0], '/');
+    if ( retval == NULL ) retval = rindex( argv[0], ']');
+    if ( retval == NULL ) retval = rindex( argv[0], '>');
+
+    return retval;
+}
 #endif
-    if ( progname == NULL )
-        progname = argv[0];
-    else
-        ++progname;
 
-    /* Check for any global args. */
-    showmessages = TRUE;
-    pm_show_version = FALSE;;
-    for ( argn = 1; argn < *argcP; ++argn ) {
-        if ( pm_keymatch( argv[argn], "-quiet", 6 ) ||
-             pm_keymatch( argv[argn], "--quiet", 7 ) ) {
-            showmessages = FALSE;
-        } else if ( pm_keymatch( argv[argn], "-version", 8 ) ||
-                    pm_keymatch( argv[argn], "--version", 9 ) ) 
-            pm_show_version = TRUE;
-        else
-            continue;
-        for ( i = argn + 1; i <= *argcP; ++i )
-            argv[i - 1] = argv[i];
-        --(*argcP);
-        }
-/* Set the stdin and stdout mode to binary.  This means nothing on Unix,
-   but matters on Windows.
 
-   Note that stdin and stdout aren't necessarily image files.  In
-   particular, stdout is sometimes text for human consumption,
-   typically printed on the terminal.  Binary mode isn't really
-   appropriate for that case.  We do this setting here without
-   any knowledge of how stdin and stdout are being used because it is
-   easy.  But we do make an exception for the case that we know the
-   file is a terminal, to get a little closer to doing the right
-   thing.  
-*/
+
+void
+pm_init(const char * const progname, unsigned int const flags) {
+/*----------------------------------------------------------------------------
+   Initialize static variables that Netpbm library routines use.
+
+   Any user of Netpbm library routines is expected to call this at the
+   beginning of this program, before any other Netpbm library routines.
+
+   A program may call this via pm_proginit() instead, though.
+-----------------------------------------------------------------------------*/
+    pm_setMessage(FALSE, NULL);
+
+    pm_progname = progname;
+    
 #ifdef O_BINARY
 #ifdef HAVE_SETMODE
+    /* Set the stdin and stdout mode to binary.  This means nothing on Unix,
+       but matters on Windows.
+       
+       Note that stdin and stdout aren't necessarily image files.  In
+       particular, stdout is sometimes text for human consumption,
+       typically printed on the terminal.  Binary mode isn't really
+       appropriate for that case.  We do this setting here without
+       any knowledge of how stdin and stdout are being used because it is
+       easy.  But we do make an exception for the case that we know the
+       file is a terminal, to get a little closer to doing the right
+       thing.  
+    */
     if (!isatty(0)) setmode(0,O_BINARY);  /* Standard Input */
     if (!isatty(1)) setmode(1,O_BINARY);  /* Standard Output */
 #endif /* HAVE_SETMODE */
 #endif /* O_BINARY */
 
-    if (pm_show_version) {
-        pm_message( "Using libpbm from Netpbm Version: %s", NETPBM_VERSION );
+}
+
+
+
+static void
+showVersion(void) {
+    pm_message( "Using libpbm from Netpbm Version: %s", NETPBM_VERSION );
 #if defined(COMPILE_TIME) && defined(COMPILED_BY)
-        pm_message( "Compiled %s by user \"%s\"",
-                    COMPILE_TIME, COMPILED_BY );
+    pm_message( "Compiled %s by user \"%s\"",
+                COMPILE_TIME, COMPILED_BY );
 #endif
 #ifdef BSD
-        pm_message( "BSD defined" );
+    pm_message( "BSD defined" );
 #endif /*BSD*/
 #ifdef SYSV
 #ifdef VMS
-        pm_message( "VMS & SYSV defined" );
+    pm_message( "VMS & SYSV defined" );
 #else
-        pm_message( "SYSV defined" );
+    pm_message( "SYSV defined" );
 #endif
 #endif /*SYSV*/
 #ifdef MSDOS
-        pm_message( "MSDOS defined" );
+    pm_message( "MSDOS defined" );
 #endif /*MSDOS*/
 #ifdef AMIGA
-        pm_message( "AMIGA defined" );
+    pm_message( "AMIGA defined" );
 #endif /* AMIGA */
-#ifdef PBMPLUS_BROKENPUTC1
-        pm_message( "PBMPLUS_BROKENPUTC1 defined" );
-#endif /*PBMPLUS_BROKENPUTC1*/
-#ifdef PBMPLUS_BROKENPUTC2
-        pm_message( "PBMPLUS_BROKENPUTC2 defined" );
-#endif /*PBMPLUS_BROKENPUTC2*/
-#ifdef PPM_PACKCOLORS
-        pm_message( "PPM_PACKCOLORS defined" );
-#endif /*PPM_PACKCOLORS*/
-#ifdef DEBUG
-        pm_message( "DEBUG defined" );
-#endif /*DEBUG*/
-#ifdef NEED_VFPRINTF1
-        pm_message( "NEED_VFPRINTF1 defined" );
-#endif /*NEED_VFPRINTF1*/
-#ifdef NEED_VFPRINTF2
-        pm_message( "NEED_VFPRINTF2 defined" );
-#endif /*NEED_VFPRINTF2*/
+    {
+        const char * rgbdef;
         pm_message( "RGB_ENV='%s'", RGBENV );
         rgbdef = getenv(RGBENV);
         if( rgbdef )
@@ -362,62 +369,108 @@ pm_init(int * const argcP, char * argv[]) {
                         RGBENV, rgbdef );
         else
             pm_message( "RGBENV= '%s' (env vbl is unset)", RGBENV);
-        exit( 0 );
     }
 }
 
-/* Error handling. */
+
+
+static void
+showNetpbmHelp(const char progname[]) {
+/*----------------------------------------------------------------------------
+  Tell the user where to get help for this program, assuming it is a Netpbm
+  program (a program that comes with the Netpbm package, as opposed to a 
+  program that just uses the Netpbm libraries).
+
+  As we are in debian, everything is in the manpages or /usr/share/
+-----------------------------------------------------------------------------*/
+   pm_message("For more help see man %s or"
+		      "the files in /usr/share/doc/netpbm*", progname);
+}
+
+
 
 void
-pm_usage( const char usage[] ) {
-    fprintf( stderr, "usage:  %s %s\n", progname, usage );
-    exit( 1 );
-    }
+pm_proginit(int * const argcP, char * argv[]) {
+/*----------------------------------------------------------------------------
+   Do various initialization things that all programs in the Netpbm package,
+   and programs that emulate such programs, should do.
 
-void
-pm_perror(const char reason[] ) {
-    const char* e;
+   This includes processing global options.
 
-#ifdef A_STRERROR
-    e = strerror(errno);
-#else /* A_STRERROR */
-    e = sys_errlist[errno];
-#endif /* A_STRERROR */
-
-    if ( reason != 0 && reason[0] != '\0' )
-        pm_error( "%s - %s", reason, e );
+   This includes calling pm_init() to initialize the Netpbm libraries.
+-----------------------------------------------------------------------------*/
+    int argn, i;
+    const char * progname;
+    bool showmessages;
+    bool show_version;
+        /* We're supposed to just show the version information, then exit the
+           program.
+        */
+    bool show_help;
+        /* We're supposed to just tell user where to get help, then exit the
+           program.
+        */
+    
+    /* Extract program name. */
+#ifdef VMS
+    progname = vmsProgname(argcP, argv);
+#else
+    progname = strrchr( argv[0], '/');
+#endif
+    if (progname == NULL)
+        progname = argv[0];
     else
-        pm_error( "%s", e );
+        ++progname;
+
+    pm_init(progname, 0);
+
+    /* Check for any global args. */
+    showmessages = TRUE;
+    show_version = FALSE;
+    show_help = FALSE;
+    for (argn = 1; argn < *argcP; ++argn) {
+        if (pm_keymatch(argv[argn], "-quiet", 6) ||
+            pm_keymatch(argv[argn], "--quiet", 7)) 
+            showmessages = FALSE;
+        else if (pm_keymatch(argv[argn], "-version", 8) ||
+                   pm_keymatch(argv[argn], "--version", 9)) 
+            show_version = TRUE;
+        else if (pm_keymatch(argv[argn], "-help", 5) ||
+                 pm_keymatch(argv[argn], "--help", 6) ||
+                 pm_keymatch(argv[argn], "-?", 2)) 
+            show_help = TRUE;
+        else
+            continue;
+        for (i = argn + 1; i <= *argcP; ++i)
+            argv[i - 1] = argv[i];
+        --(*argcP);
     }
 
+    pm_setMessage((unsigned int) showmessages, NULL);
+
+    if (show_version) {
+        showVersion();
+        exit( 0 );
+    } else if (show_help) {
+        pm_error("Use 'man %s' for help.", progname);
+        /* If we can figure out a way to distinguish Netpbm programs from 
+           other programs using the Netpbm libraries, we can do better here.
+        */
+        if (0)
+            showNetpbmHelp(progname);
+        exit(0);
+    }
+}
+
+
 void
-pm_message(const char format[], ... ) {
+pm_setMessage(int const newState, int * const oldStateP) {
+    
+    if (oldStateP)
+        *oldStateP = pm_showmessages;
 
-    va_list args;
-
-    va_start( args, format );
-
-    if ( showmessages )
-        {
-        fprintf( stderr, "%s: ", progname );
-        (void) vfprintf( stderr, format, args );
-        fputc( '\n', stderr );
-        }
-    va_end( args );
-    }
-
-void
-pm_error(const char format[], ... ) {
-    va_list args;
-
-    va_start( args, format );
-
-    fprintf( stderr, "%s: ", progname );
-    (void) vfprintf( stderr, format, args );
-    fputc( '\n', stderr );
-    va_end( args );
-    exit( 1 );
-    }
+    pm_showmessages = !!newState;
+}
 
 
 char *
@@ -734,7 +787,7 @@ char *pm_read_unknown_size(FILE * const file, long * const nread) {
             else
                 nalloc += nalloc;
             if ((buf=realloc(buf, nalloc)) == NULL)
-                pm_error("Cannot allocate %d bytes of memory", nalloc);
+                pm_error("Cannot allocate %ld bytes of memory", nalloc);
         }
 
         if ((val = getc(file)) == EOF)
@@ -835,4 +888,53 @@ pm_check(FILE * const file, const enum pm_check_type check_type,
 }
 
 
+/*
+ *	Maths wrapping
+ */
+
+void overflow2(int a, int b)
+{
+	if(a < 0 || b < 0)
+		pm_error("object too large");
+	if(b == 0)
+		return;
+	if(a > INT_MAX / b)
+		pm_error("object too large");
+}
+
+void overflow3(int a, int b, int c)
+{
+	overflow2(a,b);
+	overflow2(a*b, c);
+}
+
+void overflow_add(int a, int b)
+{
+	if( a > INT_MAX - b)
+		pm_error("object too large");
+}
+
+void *malloc2(int a, int b)
+{
+	overflow2(a, b);
+	if(a*b == 0)
+		pm_error("Zero byte allocation");
+	return malloc(a*b);
+}
+
+void *malloc3(int a, int b, int c)
+{
+	overflow3(a, b, c);
+	if(a*b*c == 0)
+		pm_error("Zero byte allocation");
+	return malloc(a*b*c);
+}
+
+void *realloc2(void * a, int b, int c)
+{
+	overflow2(b, c);
+	if(b*c == 0)
+		pm_error("Zero byte allocation");
+	return realloc(a, b*c);
+}
 
