@@ -27,15 +27,18 @@ typedef struct {
     int   CompressionLevel;
     char  Comments[MAX_COMMENT];
 
-    unsigned char * ThumbnailPointer;  // Pointer at the thumbnail
-    unsigned ThumbnailSize;     // Size of thumbnail.
+    unsigned char * ThumbnailPointer;  /* Pointer at the thumbnail */
+    unsigned ThumbnailSize;     /* Size of thumbnail. */
 
 }ImageInfo_t;
 
 
-// Prototypes for exif.c functions.
-extern void process_EXIF (unsigned char * CharBuf, unsigned int length, 
-                          ImageInfo_t * const ImageInfoP, 
-                          const int ShowTags);
+/* Prototypes for exif.c functions. */
+
+void 
+process_EXIF (unsigned char * CharBuf, unsigned int length,
+              ImageInfo_t * ImageInfoP, int const ShowTags);
+
+void 
+ShowImageInfo(ImageInfo_t * const ImageInfoP);
 
-void ShowImageInfo(ImageInfo_t * const ImageInfoP);
