@@ -173,7 +173,7 @@ parse_command_line(const int argc, char ** argv,
     unsigned int option_def_index;
 
     int argc_parse;       /* argc, except we modify it as we parse */
-    char ** const argv_parse = malloc(argc*sizeof(char *));
+    char ** const argv_parse = malloc2(argc, sizeof(char *));
     /* argv, except we modify it as we parse */
 
     option_def_index = 0;   /* incremented by OPTENTRY */
@@ -262,7 +262,8 @@ jpeg_getc (j_decompress_ptr cinfo)
 }
 
 
-boolean
+#if 0
+static boolean
 print_text_marker (j_decompress_ptr cinfo) {
 /*----------------------------------------------------------------------------
    This is a routine that you can register with the Jpeg decompressor
@@ -335,6 +336,8 @@ print_text_marker (j_decompress_ptr cinfo) {
     
     return TRUE;
 }
+#endif
+
 
 
 static void
@@ -731,7 +734,8 @@ save_exif(struct jpeg_decompress_struct const cinfo,
     if (markerP) {
         pm_writebigshort(exif_file, markerP->data_length+2);
         if (ferror(exif_file))
-            pm_error("Write of Exif header to %s failed on first byte.");
+            pm_error("Write of Exif header to %s failed on first byte.",
+                     exif_filespec);
         else {
             int rc;
 
@@ -745,7 +749,7 @@ save_exif(struct jpeg_decompress_struct const cinfo,
         /* There is no Exif header in the image */
         pm_writebigshort(exif_file, 0);
         if (ferror(exif_file))
-            pm_error("Write of Exif header file '%s' failed.");
+            pm_error("Write of Exif header file '%s' failed.", exif_filespec);
     }
     pm_close(exif_file);
 }
@@ -842,6 +846,8 @@ main(int argc, char **argv) {
   /* Calculate output image dimensions so we can allocate space */
   jpeg_calc_output_dimensions(&cinfo);
 
+  overflow2(cinfo.output_width, cinfo.output_components);
+
   jpegbuffer = ((*cinfo.mem->alloc_sarray)
                 ((j_common_ptr) &cinfo, JPOOL_IMAGE,
                  cinfo.output_width * cinfo.output_components, (JDIMENSION) 1)
