@@ -131,8 +131,7 @@ computeWeights(xel ** const cxels, int const ccols, int const crows,
     default:
         if (gsum < 0.9 || gsum > 1.1)
             pm_message("WARNING - this convolution matrix is biased.  "
-                       "average weight = %f ", 
-                       "(unbiased would be 1)",
+                       "average weight = %f (unbiased would be 1)",
                        gsum);
         break;
     }
