@@ -6,121 +6,152 @@
  * <tetsujin@sourceforge.net>
  *  and Bill Janssen <bill@janssen.org>
  *
- * Based on ppmtopuzz.c by Jef Paskanzer, from the netpbm-1mar1994 package.
+ * Based on ppmtopuzz.c by Jef Poskanzer, from the netpbm-1mar1994 package.
  */
 
+#include <string.h>
 #include <stdio.h>
 #include "pnm.h"
 #include "palm.h"
 
-int main( int argc, char **argv ) {
-    FILE* ifp;
-    pixel** pixels;
-    pixel transcolor;
-    register unsigned char *outptr;
-    unsigned char *rowdata = 0, *lastrow = 0;
-    int rows, cols, row, rowbytes, transindex = 0;
-    xelval maxval;
-    char *transparentValue = 0;
-    unsigned char outbyte, outbit;
-    unsigned char outbytes[8];
-    int format = 0;
-    int setOffset = 0;
-    int cc, max_bpp = 0, custom_colormap = 0, flags = 0, specified_bpp = 0;
-    int bpp;
-    int scanline_compression = 0, rle_compression = 0;
-    int argsGood = 1, verbose = 0, version, maxmaxval, directColor = 0;
-    unsigned int newMaxVal, color, last_color, repeatcount, limit;
-    Color_s temp_color;
-    register Color found;
-    Colormap colormap = 0;
+struct cmdline_info {
+    /* All the information the user supplied in the command line,
+       in a form easy for the program to use.
+    */
+    char *inputFilespec;  /* Filespecs of input files */
+    char *transparent;    /* -transparent value.  Null if unspec */
+    unsigned int depth;     /* -depth value.  0 if unspec */
+    unsigned int maxdepth;     /* -maxdepth value.  0 if unspec */
+    unsigned int rle_compression;
+    unsigned int scanline_compression;
+    unsigned int verbose;
+    unsigned int colormap;
+    unsigned int offset;
+};
 
-    /* Parse default params */
-    ppm_init( &argc, argv );
 
-    for (cc = 1; (cc < argc) && (argv[cc][0] == '-'); cc++) {
-        if ((strcmp(argv[cc], "-depth") == 0) && ((cc + 1) < argc)) {
-            cc++;
-            specified_bpp = atoi(argv[cc]);
-            if ((specified_bpp != 1) && (specified_bpp != 2) 
-                && (specified_bpp != 4) && (specified_bpp != 8) 
-                && (specified_bpp != 16))
-                pm_error("bad value for -depth flag; "
-                         "valid values are 1, 2, 4, 8, or 16");
-        } else if ((strcmp(argv[cc], "-maxdepth") == 0) && ((cc + 1) < argc)) {
-            cc++;
-            max_bpp = atoi(argv[cc]);
-            if ((max_bpp != 1) && (max_bpp != 2) && 
-                (max_bpp != 4) && (max_bpp != 8) && (max_bpp != 16))
-                pm_error("bad value for -maxdepth flag; "
-                         "valid values are 1, 2, 4, 8, or 16");
-        } else if ((strcmp(argv[cc], "-transparent") == 0) 
-                   && ((cc + 1) < argc)) {
-            cc++;
-            transparentValue = argv[cc];
-        } else if (strcmp(argv[cc], "-rle-compression") == 0) {
-            rle_compression = 1;
-        } else if (strcmp(argv[cc], "-scanline-compression") == 0) {
-            scanline_compression = 1;
-        } else if (strcmp(argv[cc], "-verbose") == 0) {
-            verbose = 1;
-        } else if (strcmp(argv[cc], "-colormap") == 0) {
-            custom_colormap = 1;
-        } else if (strcmp(argv[cc], "-offset") == 0) {
-            setOffset = 1;
-        } else {
-            argsGood = 0;
-        }
-    }
-    if (!argsGood) {
-        pm_usage( "[-depth BITS-PER-PIXEL] [-maxdepth BITS-PER-PIXEL] "
-                  "[-transparent COLOR] [-colormap] [-verbose] "
-                  "[-offset] [ppmfile]" );
-    }
 
-    if (max_bpp && specified_bpp && (specified_bpp > max_bpp))
-        pm_error ("argument to -depth flag must be less than or equal to "
-                  "argument to -maxdepth flag.");
+static void
+parseCommandLine(int argc, char ** argv,
+                 struct cmdline_info *cmdlineP) {
+/*----------------------------------------------------------------------------
+   Note that the file spec array we return is stored in the storage that
+   was passed to us as the argv array.
+-----------------------------------------------------------------------------*/
+    optStruct3 opt;  /* set by OPTENT3 */
+    optEntry *option_def = malloc(100*sizeof(optStruct));
+    unsigned int option_def_index;
 
-    if ( cc < argc )
-        ifp = pm_openr( argv[cc] );
+    unsigned int transSpec, depthSpec, maxdepthSpec;
+
+    option_def_index = 0;   /* incremented by OPTENTRY */
+    OPTENT3(0, "transparent",      OPT_STRING, 
+            &cmdlineP->transparent, &transSpec, 0);
+    OPTENT3(0, "depth",            OPT_UINT, 
+            &cmdlineP->depth,       &depthSpec, 0);
+    OPTENT3(0, "maxdepth",         OPT_UINT, 
+            &cmdlineP->maxdepth,    &maxdepthSpec, 0);
+    OPTENT3(0, "rle_compression",  OPT_FLAG, 
+            NULL,                   &cmdlineP->rle_compression, 0);
+    OPTENT3(0, "scanline_compression", OPT_FLAG, 
+            NULL,                   &cmdlineP->scanline_compression, 0);
+    OPTENT3(0, "verbose",          OPT_FLAG, 
+            NULL,                   &cmdlineP->verbose, 0);
+    OPTENT3(0, "colormap",         OPT_FLAG, 
+            NULL,                   &cmdlineP->colormap, 0);
+    OPTENT3(0, "offset",           OPT_FLAG, 
+            NULL,                   &cmdlineP->offset, 0);
+
+    opt.opt_table = option_def;
+    opt.short_allowed = FALSE; /* We have some short (old-fashioned) options */
+    opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
+
+    pm_optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
+        /* Uses and sets argc, argv, and some of *cmdline_p and others. */
+
+    if (depthSpec) {
+        if (cmdlineP->depth != 1 && cmdlineP->depth != 2 
+            && cmdlineP->depth != 4 && cmdlineP->depth != 8
+            && cmdlineP->depth != 16)
+            pm_error("invalid value for -depth: %u.  Valid values are "
+                     "1, 2, 4, 8, and 16", cmdlineP->depth);
+    } else
+        cmdlineP->depth = 0;
+    if (maxdepthSpec) {
+        if (cmdlineP->maxdepth != 1 && cmdlineP->maxdepth != 2 
+            && cmdlineP->maxdepth != 4 && cmdlineP->maxdepth != 8
+            && cmdlineP->maxdepth != 16)
+            pm_error("invalid value for -maxdepth: %u.  Valid values are "
+                     "1, 2, 4, 8, and 16", cmdlineP->maxdepth);
+    } else
+        cmdlineP->maxdepth = 0;
+
+    if (depthSpec && maxdepthSpec && 
+        cmdlineP->depth > cmdlineP->maxdepth)
+        pm_error("-depth value (%u) is greater than -maxdepth (%u) value.",
+                 cmdlineP->depth, cmdlineP->maxdepth);
+
+    if (!transSpec)
+        cmdlineP->transparent = NULL;
+
+    if (argc-1 > 1)
+        pm_error("This program takes at most 1 argument: the file name.  "
+                 "You specified %d", argc-1);
+    else if (argc-1 > 0) 
+        cmdlineP->inputFilespec = argv[1];
     else
-        ifp = stdin;
+        cmdlineP->inputFilespec = "-";
+}
 
-    pixels = pnm_readpnm (ifp, &cols, &rows, &maxval, &format);
-    pm_close(ifp);
+
+
+static void
+determinePalmFormat(int const cols, int const rows, 
+                    pixval const maxval, int const format, 
+                    pixel ** const pixels,
+                    unsigned int const specified_bpp,
+                    unsigned int const max_bpp, bool const custom_colormap,
+                    bool const verbose,
+                    int * const bppP, int * const maxmaxvalP, 
+                    bool * const directColorP, Colormap * const colormapP) {
 
     if (PNM_FORMAT_TYPE(format) == PBM_TYPE) {
-        /* we can always handle this one */
-        if (verbose)
-            pm_message("input is binary %dx%d", cols, rows);
+        if (custom_colormap)
+            pm_error("You specified -colormap with a black and white input "
+                     "image.  -colormap is valid only with color.");
         if (specified_bpp)
-            bpp = specified_bpp;
+            *bppP = specified_bpp;
         else
-            bpp = 1;    /* no point in wasting bits */
-        maxmaxval = 1;
-
+            *bppP = 1;    /* no point in wasting bits */
+        *maxmaxvalP = 1;
+        *directColorP = FALSE;
+        *colormapP = NULL;
+        if (verbose)
+            pm_message("output is black and white");
     } else if (PNM_FORMAT_TYPE(format) == PGM_TYPE) {
         /* we can usually handle this one, but may not have enough
            pixels.  So check... */
+        if (custom_colormap)
+            pm_error("You specified -colormap with a black and white input"
+                     "image.  -colormap is valid only with color.");
         if (specified_bpp)
-            bpp = specified_bpp;
+            *bppP = specified_bpp;
         else if (max_bpp && (maxval >= (1 << max_bpp)))
-            bpp = max_bpp;
+            *bppP = max_bpp;
         else if (maxval > 16)
-            bpp = 4;
+            *bppP = 4;
         else {
             /* scale to minimum number of bpp needed */
-            for (bpp = 1;  (1 << bpp) < maxval;  bpp *= 2)
+            for (*bppP = 1;  (1 << *bppP) < maxval;  *bppP *= 2)
                 ;
         }
-        if (bpp > 4)
-            bpp = 4;
+        if (*bppP > 4)
+            *bppP = 4;
         if (verbose)
-            pm_message("input is grayscale %dx%d, maxval %d -- "
-                       "using %d bits-per-pixel", cols, rows, maxval, bpp);
-        maxmaxval = PGM_OVERALLMAXVAL;
-
+            pm_message("output is grayscale %d bits-per-pixel", *bppP);
+        *maxmaxvalP = PGM_OVERALLMAXVAL;
+        *directColorP = FALSE;
+        *colormapP = NULL;
     } else if (PNM_FORMAT_TYPE(format) == PPM_TYPE) {
 
         /* We assume that we only get a PPM if the image cannot be
@@ -136,70 +167,146 @@ int main( int argc, char **argv ) {
         if ((specified_bpp == 16) || 
             (specified_bpp == 0 && max_bpp == 16)) {
             /* we do the 16-bit direct color */
-            directColor = 1;
-            bpp = 16;
+            *directColorP = TRUE;
+            *colormapP = NULL;
+            *bppP = 16;
         } else if (!custom_colormap) {
             /* standard indexed 8-bit color */
-            colormap = palmcolor_build_default_8bit_colormap();
-            bpp = 8;
+            *colormapP = palmcolor_build_default_8bit_colormap();
+            *bppP = 8;
             if (((specified_bpp != 0) && (specified_bpp != 8)) ||
                 ((max_bpp != 0) && (max_bpp < 8)))
                 pm_error("Must use depth of 8 for color pixmap without "
                          "custom color table.");
+            *directColorP = FALSE;
             if (verbose)
-                pm_message("input is color %dx%d, "
-                           "using default colormap at 8 bpp", cols, rows);
+                pm_message("Output is color with default colormap at 8 bpp");
         } else {
             /* indexed 8-bit color with a custom colormap */
-            colormap = 
+            *colormapP = 
                 palmcolor_build_custom_8bit_colormap(rows, cols, pixels);
-            for (bpp = 1;  (1 << bpp) < colormap->ncolors;  bpp *= 2)
-                ;
+            for (*bppP = 1; (1 << *bppP) < (*colormapP)->ncolors; *bppP *= 2);
             if (specified_bpp != 0) {
-                if (specified_bpp >= bpp)
-                    bpp = specified_bpp;
+                if (specified_bpp >= *bppP)
+                    *bppP = specified_bpp;
                 else
                     pm_error("Too many colors for specified depth.  "
                              "Use ppmquant to reduce.");
-            } else if ((max_bpp != 0) && (max_bpp < bpp)) {
+            } else if ((max_bpp != 0) && (max_bpp < *bppP)) {
                 pm_error("Too many colors for specified max depth.  "
                          "Use ppmquant to reduce.");
             }
+            *directColorP = FALSE;
             if (verbose)
-                pm_message("input is color %dx%d, using custom colormap "
+                pm_message("Output is color with custom colormap "
                            "with %d colors at %d bpp", 
-                           cols, rows, colormap->ncolors, bpp);
+                           (*colormapP)->ncolors, *bppP);
         }
-        maxmaxval = PPM_OVERALLMAXVAL;
+        *maxmaxvalP = PPM_OVERALLMAXVAL;
     } else {
         pm_error("unknown format 0x%x on input file\n", (unsigned) format);
     }
+}
+
+
+static const char * 
+formatName(int const format) {
+    
+    const char * retval;
+
+    switch(PNM_FORMAT_TYPE(format)) {
+    case PBM_TYPE: retval = "black and white"; break;
+    case PGM_TYPE: retval = "grayscale";       break;
+    case PPM_TYPE: retval = "color";           break;
+    default:       retval = "???";             break;
+    }
+    return retval;
+}
+
+        
+
+static void
+findTransparentColor(char *         const colorSpec, 
+                     pixval         const newMaxVal,
+                     bool           const directColor, 
+                     pixval         const maxval, 
+                     Colormap       const colormap,
+                     pixel *        const transcolorP, 
+                     unsigned int * const transindexP) {
+
+    *transcolorP = ppm_parsecolor(colorSpec, maxval);
+    if (!directColor) {
+        Color_s const temp_color = 
+            ((((PPM_GETR(*transcolorP)*newMaxVal) / maxval) << 16) 
+             | (((PPM_GETG(*transcolorP)*newMaxVal) / maxval) << 8)
+             | ((PPM_GETB(*transcolorP)*newMaxVal) / maxval));
+        Color const found = 
+            (bsearch(&temp_color,
+                     colormap->color_entries, colormap->ncolors,
+                     sizeof(Color_s), palmcolor_compare_colors));
+        if (!found) {
+            pm_error("Specified transparent color %s not found "
+                     "in colormap.", colorSpec);
+        } else
+            *transindexP = (*found >> 24) & 0xFF;
+    }
+}
+
+
+
+int 
+main( int argc, char **argv ) {
+    struct cmdline_info cmdline; 
+    FILE* ifp;
+    pixel** pixels;
+    pixel transcolor;
+    unsigned int transindex;
+    register unsigned char *outptr;
+    unsigned char *rowdata = 0, *lastrow = 0;
+    int rows, cols, row, rowbytes;
+    xelval maxval;
+    unsigned char outbyte, outbit;
+    unsigned char outbytes[8];
+    int format = 0;
+    int bpp;
+    int version, maxmaxval, directColor = 0;
+    unsigned int newMaxVal, color, last_color, repeatcount, limit;
+    Colormap colormap = 0;
+    unsigned short flags;
+
+    /* Parse default params */
+    ppm_init( &argc, argv );
+
+    parseCommandLine(argc, argv, &cmdline);
+
+    ifp = pm_openr(cmdline.inputFilespec);
+
+    pixels = pnm_readpnm (ifp, &cols, &rows, &maxval, &format);
+    pm_close(ifp);
+
+    if (cmdline.verbose)
+        pm_message("Input is %dx%d %s, maxval %d", 
+                   cols, rows, formatName(format), maxval);
+    
+    determinePalmFormat(cols, rows, maxval, format, pixels, 
+                        cmdline.depth, cmdline.maxdepth, cmdline.colormap, 
+                        cmdline.verbose, 
+                        &bpp, &maxmaxval, &directColor, &colormap);
 
     newMaxVal = (1 << bpp) - 1;
 
-    if (transparentValue != 0) {
-        transcolor = ppm_parsecolor (transparentValue, maxmaxval);
-        if (!directColor) {
-            temp_color = ((((PPM_GETR(transcolor)*newMaxVal) / maxval) << 16) 
-                          | (((PPM_GETG(transcolor)*newMaxVal) / maxval) << 8)
-                          | ((PPM_GETB(transcolor)*newMaxVal) / maxval));
-            found = (bsearch (&temp_color,
-                              colormap->color_entries, colormap->ncolors,
-                              sizeof(Color_s), palmcolor_compare_colors));
-            if (!found) {
-                pm_error("Specified transparent color %s not found "
-                         "in colormap.", transparentValue);
-            } else
-                transindex = (*found >> 24) & 0xFF;
-        }
-    }
+    if (cmdline.transparent) 
+        findTransparentColor(cmdline.transparent, newMaxVal, directColor,
+                             maxval, colormap, &transcolor, &transindex);
+    else 
+        transindex = 0;
 
     flags = 0;
-    if (transparentValue != 0)
+    if (cmdline.transparent)
         flags |= PALM_HAS_TRANSPARENCY_FLAG;
-    if (custom_colormap)
+    if (cmdline.colormap)
         flags |= PALM_HAS_COLORMAP_FLAG;
-    if (rle_compression || scanline_compression)
+    if (cmdline.rle_compression || cmdline.scanline_compression)
         flags |= PALM_IS_COMPRESSED_FLAG;
     if (directColor)
         flags |= PALM_DIRECT_COLOR;
@@ -207,7 +314,23 @@ int main( int argc, char **argv ) {
     /* Write Tbmp header. */
     (void) pm_writebigshort( stdout, cols );    /* width */
     (void) pm_writebigshort( stdout, rows );    /* height */
-    rowbytes = ((cols + (16 / bpp -1)) / (16 / bpp)) * 2;    
+	/* 
+     * We want to say:
+	 * rowbytes = ((cols + (16 / bpp -1)) / (16 / bpp)) * 2;
+	 * 
+	 * With fractions this is equally to 
+	 * rowbytes = ((cols - 1) * bpp/16 + 1)*2
+	 *          =  (cols - 1) * bpp/8  + 2.
+	 * When using integers, bpp/8 is at maximum less than 1 off the real value.
+	 * This gives that (speaking integerly now) (cols - 1) * (bpp/8+1) + 2
+	 * is not smaller, to which cols * (bpp/8+1) + 2 is an upper bound.
+	 *
+	 * So we do the overflow checking on this, and then execute the normal code
+	 */
+	 overflow2(cols,(bpp/8+1));
+	 overflow_add(cols*(bpp/8+1),2);
+	
+	 rowbytes = ((cols + (16 / bpp -1)) / (16 / bpp)) * 2;
         /* bytes per row - always a word boundary */
     pm_writebigshort(stdout, rowbytes);
     (void) pm_writebigshort( stdout, flags );
@@ -216,13 +339,13 @@ int main( int argc, char **argv ) {
     /* we need to mark this as version 1 if we use more than 1 bpp,
        version 2 if we use compression or transparency */
 
-    version = ((transparentValue != 0)
-               || rle_compression
-               || scanline_compression) ? 
-        2 : (((bpp > 1)||(custom_colormap)) ? 1 : 0);
+    version = ((cmdline.transparent)
+               || cmdline.rle_compression
+               || cmdline.scanline_compression) ? 
+        2 : (((bpp > 1)||(cmdline.colormap)) ? 1 : 0);
     fputc(version, stdout);
 
-    if (setOffset) {
+    if (cmdline.offset) {
         /* Offset is measured in double-words. Those are 4-byte, right? */
         /* Account for bitmap data size, plus header size, and round up */
         pm_writebigshort(stdout, (rowbytes * rows + 16 + 3)/4);
@@ -231,9 +354,9 @@ int main( int argc, char **argv ) {
     }
 
     fputc(transindex, stdout);  /* transparent index */
-    if (rle_compression)
+    if (cmdline.rle_compression)
         fputc(PALM_COMPRESSION_RLE, stdout);
-    else if (scanline_compression)
+    else if (cmdline.scanline_compression)
         fputc(PALM_COMPRESSION_SCANLINE, stdout);
     else
         fputc(PALM_COMPRESSION_NONE, stdout);
@@ -241,7 +364,10 @@ int main( int argc, char **argv ) {
     (void) pm_writebigshort( stdout, 0 );   /* reserved by Palm */
 
     /* if there's a colormap, write it out */
-    if (custom_colormap) {
+    if (cmdline.colormap) {
+        if (!colormap)
+            pm_error("Internal error: user specified -colormap, but we did "
+                     "not generate a colormap.");
         qsort (colormap->color_entries, colormap->ncolors,
                sizeof(Color_s), palmcolor_compare_indices);
         pm_writebigshort( stdout, colormap->ncolors );
@@ -261,7 +387,7 @@ int main( int argc, char **argv ) {
             pm_error("Don't know how to create %d bit DirectColor bitmaps.", 
                      bpp);
         }
-        if (transparentValue != 0) {
+        if (cmdline.transparent) {
             fputc(0, stdout);
             fputc((PPM_GETR(transcolor) * 255) / maxval, stdout);
             fputc((PPM_GETG(transcolor) * 255) / maxval, stdout);
@@ -273,7 +399,7 @@ int main( int argc, char **argv ) {
 
     last_color = 0xFFFFFFFF;
     rowdata = malloc(rowbytes);
-    if (scanline_compression)
+    if (cmdline.scanline_compression)
         lastrow = malloc(rowbytes);
     /* And write out the data. */
     for ( row = 0; row < rows; ++row ) {
@@ -309,15 +435,15 @@ int main( int argc, char **argv ) {
                     color = (newMaxVal - color); 
                         /* note grayscale maps are inverted */
                 } else {
-                    temp_color = 
+                    Color_s const temp_color =
                         ((((PPM_GETR(pixels[row][col])*newMaxVal)/maxval)<<16) 
                          | (((PPM_GETG(pixels[row][col])*newMaxVal)/maxval)<<8)
                          | (((PPM_GETB(pixels[row][col])*newMaxVal)/maxval)));
-                    found = (bsearch (&temp_color,
-                                      colormap->color_entries, 
-                                      colormap->ncolors,
-                                      sizeof(Color_s), 
-                                      palmcolor_compare_colors));
+                    Color const found = (bsearch (&temp_color,
+                                                  colormap->color_entries, 
+                                                  colormap->ncolors,
+                                                  sizeof(Color_s), 
+                                                  palmcolor_compare_colors));
                     if (!found) {
                         pm_error("Color %d:%d:%d not found in colormap.  "
                                  "Try using ppmquant to reduce the "
@@ -348,7 +474,7 @@ int main( int argc, char **argv ) {
         }
         /* now output the row of pixels, compressed appropriately */
 
-        if (scanline_compression) {
+        if (cmdline.scanline_compression) {
             int col;
             for (col = 0;  col < rowbytes;  col += 8) {
                 limit = ((rowbytes - col) < 8) ? (rowbytes - col) : 8;
@@ -366,7 +492,7 @@ int main( int argc, char **argv ) {
             }
             memcpy (lastrow, rowdata, rowbytes);
       
-        } else if (rle_compression) {
+        } else if (cmdline.rle_compression) {
             int col;
             /* we output a count of the number of bytes a value is
                repeated, followed by that byte value 
@@ -389,7 +515,7 @@ int main( int argc, char **argv ) {
         }
     }
     /* If we've set the offset, pad to the next doubleword */
-    if (setOffset) {
+    if (cmdline.offset) {
         int cc;
 
         for(cc = (rowbytes * rows + 16)%4; cc > 0; cc--) {
