@@ -12,9 +12,12 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <sys/wait.h>   /* For popen() exit codes */
+#include <signal.h>
+#include <sys/wait.h>  
 #include <sys/stat.h>
+
 #include "pnm.h"
+#include "shhopt.h"
 
 enum orientation {PORTRAIT, LANDSCAPE, UNSPECIFIED};
 struct box {
@@ -34,11 +37,11 @@ struct cmdline_info {
        in a form easy for the program to use.
     */
     char *input_filespec;  /* Filespecs of input files */
-    int forceplain;
+    unsigned int forceplain;
     struct box extract_box;
-    int nocrop;
-    int format_type;
-    int verbose;
+    unsigned int nocrop;
+    unsigned int format_type;
+    unsigned int verbose;
     float xborder;
     int xmax;
     int xsize;
@@ -46,7 +49,7 @@ struct cmdline_info {
     int ymax;
     int ysize;
     enum orientation orientation;
-    int goto_stdout;
+    unsigned int goto_stdout;
 } cmdline;
 
 
@@ -57,44 +60,40 @@ parse_command_line(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
+    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+        /* Instructions to optParseOptions3 on how to parse our options.
          */
-    optStruct2 opt;
+    optStruct3 opt;
 
     unsigned int option_def_index;
 
-    int pbm_opt, pgm_opt, ppm_opt;
-    int portrait_opt, landscape_opt;
+    unsigned int pbm_opt, pgm_opt, ppm_opt;
+    unsigned int portrait_opt, landscape_opt;
     float llx, lly, urx, ury;
+    unsigned int llxSpec, llySpec, urxSpec, urySpec;
     
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0,   "forceplain", OPT_FLAG,   &cmdline_p->forceplain,     0);
-    OPTENTRY(0,   "llx",        OPT_FLOAT,  &llx,                       0);
-    OPTENTRY(0,   "lly",        OPT_FLOAT,  &lly,                       0);
-    OPTENTRY(0,   "urx",        OPT_FLOAT,  &urx,                       0);
-    OPTENTRY(0,   "ury",        OPT_FLOAT,  &ury,                       0);
-    OPTENTRY(0,   "nocrop",     OPT_FLAG,   &cmdline_p->nocrop,         0);
-    OPTENTRY(0,   "pbm",        OPT_FLAG,   &pbm_opt,                   0);
-    OPTENTRY(0,   "pgm",        OPT_FLAG,   &pgm_opt,                   0);
-    OPTENTRY(0,   "ppm",        OPT_FLAG,   &ppm_opt,                   0);
-    OPTENTRY(0,   "verbose",    OPT_FLAG,   &cmdline_p->verbose,        0);
-    OPTENTRY(0,   "xborder",    OPT_FLOAT,  &cmdline_p->xborder,        0);
-    OPTENTRY(0,   "xmax",       OPT_UINT,   &cmdline_p->xmax,           0);
-    OPTENTRY(0,   "xsize",      OPT_UINT,   &cmdline_p->xsize,          0);
-    OPTENTRY(0,   "yborder",    OPT_FLOAT,  &cmdline_p->yborder,        0);
-    OPTENTRY(0,   "ymax",       OPT_UINT,   &cmdline_p->ymax,           0);
-    OPTENTRY(0,   "ysize",      OPT_UINT,   &cmdline_p->ysize,          0);
-    OPTENTRY(0,   "portrait",   OPT_FLAG,   &portrait_opt,              0);
-    OPTENTRY(0,   "landscape",  OPT_FLAG,   &landscape_opt,             0);
-    OPTENTRY(0,   "stdout",     OPT_FLAG,   &cmdline_p->goto_stdout,    0);
+    OPTENT3(0, "forceplain", OPT_FLAG,  NULL, &cmdline_p->forceplain,     0);
+    OPTENT3(0, "llx",        OPT_FLOAT, &llx, &llxSpec,                   0);
+    OPTENT3(0, "lly",        OPT_FLOAT, &lly, &llySpec,                   0);
+    OPTENT3(0, "urx",        OPT_FLOAT, &urx, &urxSpec,                   0);
+    OPTENT3(0, "ury",        OPT_FLOAT, &ury, &urySpec,                   0);
+    OPTENT3(0, "nocrop",     OPT_FLAG,  NULL, &cmdline_p->nocrop,         0);
+    OPTENT3(0, "pbm",        OPT_FLAG,  NULL, &pbm_opt,                   0);
+    OPTENT3(0, "pgm",        OPT_FLAG,  NULL, &pgm_opt,                   0);
+    OPTENT3(0, "ppm",        OPT_FLAG,  NULL, &ppm_opt,                   0);
+    OPTENT3(0, "verbose",    OPT_FLAG,  NULL, &cmdline_p->verbose,        0);
+    OPTENT3(0, "xborder",    OPT_FLOAT, &cmdline_p->xborder, NULL,        0);
+    OPTENT3(0, "xmax",       OPT_UINT,  &cmdline_p->xmax, NULL,           0);
+    OPTENT3(0, "xsize",      OPT_UINT,  &cmdline_p->xsize, NULL,          0);
+    OPTENT3(0, "yborder",    OPT_FLOAT, &cmdline_p->yborder, NULL,        0);
+    OPTENT3(0, "ymax",       OPT_UINT,  &cmdline_p->ymax, NULL,           0);
+    OPTENT3(0, "ysize",      OPT_UINT,  &cmdline_p->ysize, NULL,          0);
+    OPTENT3(0, "portrait",   OPT_FLAG,  NULL, &portrait_opt,              0);
+    OPTENT3(0, "landscape",  OPT_FLAG,  NULL, &landscape_opt,             0);
+    OPTENT3(0, "stdout",     OPT_FLAG,  NULL, &cmdline_p->goto_stdout,    0);
 
     /* Set the defaults */
-    cmdline_p->forceplain = FALSE;
-    cmdline_p->nocrop = FALSE;
-    cmdline_p->verbose = FALSE;
-    pbm_opt = pgm_opt = ppm_opt = FALSE;
-    portrait_opt = landscape_opt = FALSE;
     llx = lly = urx = ury = -1;
     cmdline_p->xmax = 612;
     cmdline_p->ymax = 792;
@@ -105,7 +104,7 @@ parse_command_line(int argc, char ** argv,
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    pm_optParseOptions2(&argc, argv, opt, 0);
+    pm_optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
     if (argc-1 == 0)
@@ -137,14 +136,14 @@ parse_command_line(int argc, char ** argv,
     /* If any one of the 4 bounding box coordinates is given on the
        command line, we default any of the 4 that aren't.  
     */
-    if (llx != -1 || lly != -1 || urx != -1 || ury != -1) {
-        if (llx == -1) cmdline_p->extract_box.llx = 72;
+    if (llxSpec || llySpec || urxSpec || urySpec) {
+        if (!llxSpec) cmdline_p->extract_box.llx = 72;
         else cmdline_p->extract_box.llx = llx * 72;
-        if (lly == -1) cmdline_p->extract_box.lly = 72;
+        if (!llySpec) cmdline_p->extract_box.lly = 72;
         else cmdline_p->extract_box.lly = lly * 72;
-        if (urx == -1) cmdline_p->extract_box.urx = 540;
+        if (!urxSpec) cmdline_p->extract_box.urx = 540;
         else cmdline_p->extract_box.urx = urx * 72;
-        if (ury == -1) cmdline_p->extract_box.ury = 720;
+        if (!urySpec) cmdline_p->extract_box.ury = 720;
         else cmdline_p->extract_box.ury = ury * 72;
     } else {
         cmdline_p->extract_box.llx = -1;
@@ -175,6 +174,7 @@ add_ps_to_filespec(const char orig_filespec[], char ** const new_filespec_p,
     else {
         char *filespec_plus_ps;
 
+        overflow_add(strlen(orig_filespec), 4);
         filespec_plus_ps = malloc(strlen(orig_filespec) + 4);
         strcpy(filespec_plus_ps, orig_filespec);
         strcat(filespec_plus_ps, ".ps");
@@ -245,6 +245,37 @@ compute_size_res(const struct cmdline_info cmdline,
 }
 
 
+enum postscript_language {COMMON_POSTSCRIPT, ENCAPSULATED_POSTSCRIPT};
+
+static enum postscript_language
+language_declaration(const char input_filespec[], int const verbose) {
+
+    FILE *infile;
+    enum postscript_language language;
+    char line[80];
+
+    infile = pm_openr(input_filespec);
+
+    if (fgets(line, sizeof(line), infile) == NULL)
+        language = COMMON_POSTSCRIPT;
+    else {
+        const char eps_header[] = "%!PS-Adobe EPSF-3.0";
+
+        if (strncmp(line, eps_header, sizeof(eps_header)))
+            language = ENCAPSULATED_POSTSCRIPT;
+        else
+            language = COMMON_POSTSCRIPT;
+    }
+    fclose(infile);
+    if (verbose)
+        pm_message("language is %s",
+                   language == ENCAPSULATED_POSTSCRIPT ?
+                   "encapsulated postscript" :
+                   "not encapsulated postscript");
+    return language;
+}
+
+
 
 static struct box
 compute_box_to_extract(const struct box cmdline_extract_box,
@@ -257,9 +288,9 @@ compute_box_to_extract(const struct box cmdline_extract_box,
         /* User told us what box to extract, so that's what we'll do */
         retval = cmdline_extract_box;
     else {
-        /* Try to get the bounding box from the Postscript %%BoundingBox
-           statement in the input.
-           */
+        /* Try to get the bounding box from the DSC %%BoundingBox
+           statement (A Postscript comment) in the input.
+        */
         struct box ps_bb;  /* Box described by %%BoundingBox stmt in input */
 
         if (strcmp(input_filespec, "-") == 0)
@@ -416,6 +447,7 @@ compute_outfile_arg(const struct cmdline_info cmdline) {
 
     char *retval;  /* malloc'ed */
 
+    overflow_add(strlen(cmdline.input_filespec), 10);
     retval = malloc(strlen(cmdline.input_filespec) + 10);
 
     if (cmdline.goto_stdout)
@@ -460,67 +492,200 @@ compute_gs_device(const int format_type, const int forceplain) {
 
 
 
+static void
+findGhostscriptProg(const char ** const retvalP) {
+    
+    *retvalP = NULL;  /* initial assumption */
+    if (getenv("GHOSTSCRIPT"))
+        *retvalP = strdup(getenv("GHOSTSCRIPT"));
+    if (*retvalP == NULL) {
+        if (getenv("PATH") != NULL) {
+            char *pathwork;  /* malloc'ed */
+            const char * candidate;
+
+            pathwork = strdup(getenv("PATH"));
+            
+            candidate = strtok(pathwork, ":");
+
+            *retvalP = NULL;
+            while (!*retvalP && candidate) {
+                struct stat statbuf;
+                const char * filename;
+                int rc;
+
+                asprintf(&filename, "%s/gs", candidate);
+                rc = stat(filename, &statbuf);
+                if (rc == 0) {
+                    if (S_ISREG(statbuf.st_mode))
+                        *retvalP = strdup(filename);
+                } else if (errno != ENOENT)
+                    pm_error("Error looking for Ghostscript program.  "
+                             "stat(\"%s\") returns errno %d (%s)",
+                             filename, errno, strerror(errno));
+		free((void *) filename);
+
+                candidate = strtok(NULL, ":");
+            }
+            free(pathwork);
+        }
+    }
+    if (*retvalP == NULL)
+        *retvalP = strdup("/usr/bin/gs");
+}
+
+
+
+static void
+execGhostscript(int const inputPipeFd,
+                const char ghostscript_device[],
+                const char outfile_arg[], 
+                int const xsize, int const ysize, 
+                int const xres, int const yres,
+                const char input_filespec[], int const verbose) {
+    
+    const char *arg0;
+    const char *ghostscriptProg;
+    const char *deviceopt;
+    const char *outfileopt;
+    const char *gopt;
+    const char *ropt;
+    int rc;
+
+    findGhostscriptProg(&ghostscriptProg);
+
+    /* Put the input pipe on Standard Input */
+    rc = dup2(inputPipeFd, STDIN_FILENO);
+    close(inputPipeFd);
+
+    asprintf(&arg0, "gs");
+    asprintf(&deviceopt, "-sDEVICE=%s", ghostscript_device);
+    asprintf(&outfileopt, "-sOutputFile=%s", outfile_arg);
+    asprintf(&gopt, "-g%dx%d", xsize, ysize);
+    asprintf(&ropt, "-r%dx%d", xres, yres);
+
+    if (verbose) {
+        pm_message("about to message");
+        pm_message("execing '%s' with args '%s' (arg 0), "
+                   "'%s', '%s', '%s', '%s', '%s', '%s', '%s'",
+                   ghostscriptProg, arg0,
+                   deviceopt, outfileopt, gopt, ropt, "-q", "-dNOPAUSE", "-");
+    }
+
+    execl(ghostscriptProg, arg0, deviceopt, outfileopt, gopt, ropt, "-q",
+          "-dNOPAUSE", "-", NULL);
+    
+    pm_error("execl() of Ghostscript ('%s') failed, errno=%d (%s)",
+             ghostscriptProg, errno, strerror(errno));
+}
+
+
+
+
 static void
 execute_ghostscript(const char pstrans[], const char ghostscript_device[],
                     const char outfile_arg[], 
                     const int xsize, const int ysize, 
                     const int xres, const int yres,
-                    const char input_filespec[], const int verbose) {
+                    const char input_filespec[], 
+                    const enum postscript_language language,
+                    const int verbose) {
 
     int gs_exit;  /* wait4 exit code from Ghostscript */
     FILE *gs;  /* Pipe to Ghostscript's standard input */
     FILE *infile;
+    int rc;
     int eof;  /* End of file on input */
-    char ghostscript_command[250];
+    int pipefd[2];
 
     if (strlen(outfile_arg) > 80)
         pm_error("output file spec too long.");
     
-    sprintf(ghostscript_command,
-            "gs -sDEVICE='%s' -sOutputFile='%s' -g'%dx%d' -r'%dx%d' "
-            "-q -dNOPAUSE -",
-            ghostscript_device, outfile_arg, 
-            xsize, ysize, xres, yres);
-
-    if (cmdline.verbose) {
-        pm_message("Shell command: %s", ghostscript_command);
-        pm_message("Postscript prefix command: '%s'", pstrans);
-    }
-
-    infile = pm_openr(input_filespec);
-
-    gs = popen(ghostscript_command, "w");
-    if (gs == NULL) 
-        pm_error("Unable to start Ghostscript shell command: %s\n",
-                 ghostscript_command);
+    rc = pipe(pipefd);
+    if (rc < 0)
+        pm_error("Unable to create pipe to talk to Ghostscript process.  "
+                 "errno = %d (%s)", errno, strerror(errno));
     
-    fprintf(gs, "%s\n", pstrans);
+    rc = fork();
+    if (rc < 0)
+        pm_error("Unable to fork a Ghostscript process.  errno=%d (%s)",
+                 errno, strerror(errno));
+    else if (rc == 0) {
+        /* Child process */
+        close(pipefd[1]);
+        execGhostscript(pipefd[0], ghostscript_device, outfile_arg,
+                        xsize, ysize, xres, yres, input_filespec, verbose);
+    } else {
+        pid_t const ghostscriptPid = rc;
+        int const pipeToGhostscriptFd = pipefd[1];
+        /* parent process */
+        close(pipefd[0]);
 
-    eof = FALSE;
-    while (!eof) {
-        char buffer[4096];
-        int bytes_read;
+        gs = fdopen(pipeToGhostscriptFd, "w");
+        if (gs == NULL) 
+            pm_error("Unable to open stream on pipe to Ghostscript process.");
+    
+        infile = pm_openr(input_filespec);
+        /*
+          In encapsulated Postscript, we the encapsulator are supposed to
+          handle showing the page (which we do by passing a showpage
+          statement to Ghostscript).  Any showpage statement in the 
+          input must be defined to have no effect.
+          
+          See "Enscapsulated PostScript Format File Specification",
+          v. 3.0, 1 May 1992, in particular Example 2, p. 21.  I found
+          it at 
+          http://partners.adobe.com/asn/developer/pdfs/tn/5002.EPSF_Spec.pdf
+          The example given is a much fancier solution than we need
+          here, I think, so I boiled it down a bit.  JM 
+        */
+        if (language == ENCAPSULATED_POSTSCRIPT)
+            fprintf(gs, "\n/b4_Inc_state save def /showpage { } def\n");
+ 
+        if (verbose) 
+            pm_message("Postscript prefix command: '%s'", pstrans);
+
+        fprintf(gs, "%s\n", pstrans);
+
+        /* If our child dies, it closes the pipe and when we next write to it,
+           we get a SIGPIPE.  We must survive that signal in order to report
+           on the fate of the child.  So we ignore SIGPIPE:
+        */
+        signal(SIGPIPE, SIG_IGN);
+
+        eof = FALSE;
+        while (!eof) {
+            char buffer[4096];
+            int bytes_read;
+            
+            bytes_read = fread(buffer, 1, sizeof(buffer), infile);
+            if (bytes_read == 0) 
+                eof = TRUE;
+            else 
+                fwrite(buffer, 1, bytes_read, gs);
+        }
+        pm_close(infile);
+
+        if (language == ENCAPSULATED_POSTSCRIPT)
+            fprintf(gs, "\nb4_Inc_state restore showpage\n");
+
+        fclose(gs);
         
-        bytes_read = fread(buffer, 1, sizeof(buffer), infile);
-        if (bytes_read == 0) 
-            eof = TRUE;
-        else 
-            fwrite(buffer, 1, bytes_read, gs);
-    }
-    
-    gs_exit = pclose(gs);
-    pm_close(infile);
+        waitpid(ghostscriptPid, &gs_exit, 0);
+        if (rc < 0)
+            pm_error("Wait for Ghostscript process to terminated failed.  "
+                     "errno = %d (%s)", errno, strerror(errno));
 
-    if (gs_exit != 0) {
-        if (WIFEXITED(gs_exit))
-            pm_error("Ghostscript command failed.  Exit code=%d\n"
-                     "Command was: %s", WEXITSTATUS(gs_exit), 
-                     ghostscript_command);
-        else if (WIFSIGNALED(gs_exit))
-            pm_error("Ghostscript process died due to a signal %d.",
-                     WTERMSIG(gs_exit));
-        else 
-            pm_error("Ghostscript process died with exit code %d", gs_exit);
+        if (gs_exit != 0) {
+            if (WIFEXITED(gs_exit))
+                pm_error("Ghostscript failed.  Exit code=%d\n", 
+                         WEXITSTATUS(gs_exit));
+            else if (WIFSIGNALED(gs_exit))
+                pm_error("Ghostscript process died due to a signal %d.",
+                         WTERMSIG(gs_exit));
+            else 
+                pm_error("Ghostscript process died with exit code %d", 
+                         gs_exit);
+        }
     }
 }
 
@@ -540,6 +705,7 @@ main(int argc, char **argv) {
     struct box bordered_box;
         /* Same as above, but expanded to include borders */
 
+    enum postscript_language language;
     enum orientation orientation;
     char *ghostscript_device;
     char *outfile_arg;
@@ -555,6 +721,8 @@ main(int argc, char **argv) {
     extract_box = compute_box_to_extract(cmdline.extract_box, input_filespec, 
                                          cmdline.verbose);
 
+    language = language_declaration(input_filespec, cmdline.verbose);
+    
     orientation = compute_orientation(cmdline, extract_box);
 
     bordered_box = add_borders(extract_box, cmdline.xborder, cmdline.yborder,
@@ -575,7 +743,7 @@ main(int argc, char **argv) {
     
     execute_ghostscript(pstrans, ghostscript_device, outfile_arg, 
                         xsize, ysize, xres, yres, input_filespec,
-                        cmdline.verbose);
+                        language, cmdline.verbose);
 
     return 0;
 }
