@@ -40,6 +40,8 @@
 **  - added IFF-PBM format
 */
 
+#include <string.h>
+
 #include "ppm.h"
 #include "ppmcmap.h"
 #include "ilbm.h"
@@ -123,7 +125,8 @@ static rawtype * alloc_rawrow ARGS((int cols));
 static ColorMap * alloc_cmap ARGS((void));
 static void check_cmap ARGS((BitMapHeader *bmhd, ColorMap *cmap));
 static void * xmalloc ARGS((int bytes));
-#define MALLOC(n, type)     (type *)xmalloc((n) * sizeof(type))
+static void * xmalloc2 ARGS((int x, int y));
+#define MALLOC(n, type)     (type *)xmalloc2((n), sizeof(type))
 
 #define FACTOR_4BIT     17      /* scale factor maxval 15 -> maxval 255 */
 
@@ -280,7 +283,8 @@ main(argc, argv)
                 skip_chunk(ifp, iffid, chunksize);
             }
             if( formsize != 0xffffffff )
-                pm_message("warning - %s chunk with FORM size 0x%08lx (should be 0x%08lx)",
+                pm_message("warning - %s chunk with FORM size 0x%08lx "
+                           "(should be 0x%08x)",
                             ID2string(iffid), formsize, 0xffffffff);
             endchunk = 1;
         }
@@ -347,6 +351,7 @@ main(argc, argv)
                 }
 
                 if( typeid == ID_ILBM ) {
+                    overflow_add(bmhd->w, 15);
                     ilbmrow = MALLOC(RowBytes(bmhd->w), unsigned char);
                     viewportmodes |= fakeviewport;      /* -isham/-isehb */
 
@@ -931,6 +936,9 @@ dcol_to_ppm(ifp, chunksize, bmhd, cmap, dcol)
     if( redmaxval != maxval || greenmaxval != maxval || bluemaxval != maxval )
         pm_message("scaling colors to %d bits", pm_maxvaltobits(maxval));
 
+    overflow_add(redmaxval, 1);
+    overflow_add(greenmaxval, 1);
+    overflow_add(bluemaxval, 1);
     redtable   = MALLOC(redmaxval  +1, pixval);
     greentable = MALLOC(greenmaxval+1, pixval);
     bluetable  = MALLOC(bluemaxval +1, pixval);
@@ -1280,6 +1288,7 @@ decode_row(ifp, chunksizeP, chunkyrow, nPlanes, bmhd)
     rawtype *chp;
 
     cols = bmhd->w;
+    overflow_add(cols, 15);
     bytes = RowBytes(cols);
     for( plane = 0; plane < nPlanes; plane++ ) {
         int mask;
@@ -1320,6 +1329,7 @@ decode_mask(ifp, chunksizeP, chunkyrow, bmhd)
         case mskNone:
             break;
         case mskHasMask:        /* mask plane */
+            overflow_add(cols, 15);
             read_ilbm_plane(ifp, chunksizeP, RowBytes(cols), bmhd->compression);
             if( maskfile ) {
                 ilp = ilbmrow;
@@ -1393,6 +1403,23 @@ xmalloc(bytes)
     return mem;
 }
 
+static void *
+xmalloc2(x, y)
+    int x;
+    int y;
+{
+    void *mem;
+
+    overflow2(x,y);
+    if( x * y == 0 )
+        return NULL;
+
+    mem = malloc2(x,y);
+    if( mem == NULL )
+        pm_error("out of memory allocating %d bytes", x * y);
+    return mem;
+}
+
 
 static char *
 ID2string(id)
@@ -1581,6 +1608,9 @@ read_pchg(ifp, iffid, chunksize, cmap)
         cmap->mp_change[i] = NULL;
     if( PCHG.StartLine < 0 ) {
         int nch;
+        if(PCHG.MaxReg < PCHG.MinReg)
+        	pm_error("assert: MinReg > MaxReg");
+        overflow_add(PCHG.MaxReg-PCHG.MinReg, 2);
         nch = PCHG.MaxReg - PCHG.MinReg +1;
         cmap->mp_init = MALLOC(nch + 1, PaletteChange);
         for( i = 0; i < nch; i++ )
@@ -1711,7 +1741,9 @@ PCHG_ConvertSmall(PCHG, cmap, mask, datasize)
             ChangeCount32 = *data++;
             datasize -= 2;
 
+	    overflow_add(ChangeCount16, ChangeCount32);
             changes = ChangeCount16 + ChangeCount32;
+            overflow_add(changes, 1);
             cmap->mp_change[row] = MALLOC(changes + 1, PaletteChange);
             for( i = 0; i < changes; i++ ) {
                 if( totalchanges >= PCHG->TotalChanges ) goto fail;
@@ -1797,6 +1829,7 @@ PCHG_ConvertBig(PCHG, cmap, mask, datasize)
             if( datasize < 2 ) goto fail;
             changes = BIG_WORD(data); data += 2; datasize -= 2;
 
+	    overflow_add(changes, 1);
             cmap->mp_change[row] = MALLOC(changes + 1, PaletteChange);
             for( i = 0; i < changes; i++ ) {
                 if( totalchanges >= PCHG->TotalChanges ) goto fail;
