@@ -20,7 +20,7 @@
  * Department of Computer Science
  * University of British Columbia
  *
- * $Id: picttoppm.c,v 1.7 1993/10/26 22:40:31 phillips Exp phillips $
+ * $Id: picttoppm.c,v 1.5 2004/01/05 10:39:56 aba-guest Exp $
  */
 
 #define _BSD_SOURCE 1
@@ -583,7 +583,7 @@ char* argv[];
 	}
 
 	if (load_fontdir("fontdir") < 0)
-		pm_message("warning: can't load font directory 'fontdir'\n", 0,0,0,0);
+		pm_message("warning: can't load font directory 'fontdir'\n");
 
 	if (argn < argc) {
 		ifp = pm_openr(argv[argn]);
@@ -625,7 +625,7 @@ interpret_pict()
 	picSize = read_word();
 
 	if (verbose)
-		pm_message("picture size = %d (0x%x)", picSize, picSize, 0, 0, 0);
+		pm_message("picture size = %d (0x%x)", picSize, picSize);
 
 	stage = "reading picture frame";
 	read_rect(&picFrame);
@@ -634,7 +634,7 @@ interpret_pict()
 		dump_rect("Picture frame:", &picFrame);
 		pm_message("Picture size is %d x %d",
 			picFrame.right - picFrame.left,
-			picFrame.bottom - picFrame.top, 0, 0, 0);
+			picFrame.bottom - picFrame.top);
 	}
 
 	if (!fullres)
@@ -659,16 +659,16 @@ interpret_pict()
 	}
 
 	if (verbose)
-		pm_message("PICT version %d", version, 0, 0, 0, 0);
+		pm_message("PICT version %d", version);
 
 	while((opcode = get_op(version)) != 0xff) {
 		if (opcode < 0xa2) {
 			if (verbose) {
 				stage = optable[opcode].name;
 				if (!strcmp(stage, "reserved"))
-					pm_message("reserved opcode=0x%x", opcode, 0, 0, 0, 0);
+					pm_message("reserved opcode=0x%x", opcode);
 				else
-					pm_message("%s", stage = optable[opcode].name, 0, 0, 0, 0);
+					pm_message("%s", stage = optable[opcode].name);
 			}
 
 			if (optable[opcode].impl != NULL)
@@ -687,42 +687,42 @@ interpret_pict()
 		}
 		else if (opcode == 0xc00) {
 			if (verbose)
-				pm_message("HeaderOp", 0, 0, 0, 0, 0);
+				pm_message("HeaderOp");
 			stage = "HeaderOp";
 			skip(24);
 		}
 		else if (opcode >= 0xa2 && opcode <= 0xaf) {
 			stage = "skipping reserved";
 			if (verbose)
-				pm_message("%s 0x%x", stage, opcode, 0, 0, 0);
+				pm_message("%s 0x%x", stage, opcode);
 			skip(read_word());
 		}
 		else if (opcode >= 0xb0 && opcode <= 0xcf) {
 			/* just a reserved opcode, no data */
 			if (verbose)
-				pm_message("reserved 0x%x", opcode, 0, 0, 0);
+				pm_message("reserved 0x%x", opcode);
 		}
 		else if (opcode >= 0xd0 && opcode <= 0xfe) {
 			stage = "skipping reserved";
 			if (verbose)
-				pm_message("%s 0x%x", stage, opcode, 0, 0, 0);
+				pm_message("%s 0x%x", stage, opcode);
 			skip(read_long());
 		}
 		else if (opcode >= 0x100 && opcode <= 0x7fff) {
 			stage = "skipping reserved";
 			if (verbose)
-				pm_message("%s 0x%x", stage, opcode, 0, 0, 0);
+				pm_message("%s 0x%x", stage, opcode);
 			skip((opcode >> 7) & 255);
 		}
 		else if (opcode >= 0x8000 && opcode <= 0x80ff) {
 			/* just a reserved opcode */
 			if (verbose)
-				pm_message("reserved 0x%x", opcode, 0, 0, 0, 0);
+				pm_message("reserved 0x%x", opcode);
 		}
 		else if (opcode >= 8100 && opcode <= 0xffff) {
 			stage = "skipping reserved";
 			if (verbose)
-				pm_message("%s 0x%x", stage, opcode, 0, 0, 0);
+				pm_message("%s 0x%x", stage, opcode);
 			skip(read_long());
 		}
 		else
@@ -908,7 +908,7 @@ do_blits()
 			rectscale(&bi->dstRect, xscale, yscale);
 
 		pm_message("Scaling output by %f in X and %f in Y",
-			xscale, yscale, 0, 0, 0);
+                   xscale, yscale);
 		rectscale(&picFrame, xscale, yscale);
 	}
 
@@ -931,7 +931,10 @@ static void
 Opcode_9A(version)
 int version;
 {
+/*
 #ifdef DUMP
+*/
+#if 0
 	FILE *fp = fopen("data", "wb");
 	int ch;
 	if (fp == NULL) exit(1);
@@ -978,7 +981,7 @@ int version;
 
 	mode = read_word();
 	if (verbose)
-		pm_message("transfer mode = %s", const_name(transfer_name, mode),0,0,0,0);
+		pm_message("transfer mode = %s", const_name(transfer_name, mode));
 
 	pm = unpackbits(&p.Bounds, 0, p.pixelSize);
 
@@ -1002,7 +1005,7 @@ int version;
 	rowBytes = read_word();
 
 	if (verbose)
-		pm_message("rowbytes = 0x%x (%d)", rowBytes, rowBytes & 0x7fff, 0, 0, 0);
+		pm_message("rowbytes = 0x%x (%d)", rowBytes, rowBytes & 0x7fff);
 
 	if (rowBytes & 0x8000)
 		do_pixmap(version, rowBytes, 0);
@@ -1043,7 +1046,7 @@ int is_region;
 	read_rect(&dstRect);
 	mode = read_word();
 	if (verbose)
-		pm_message("transfer mode = %s", const_name(transfer_name, mode),0,0,0,0);
+		pm_message("transfer mode = %s", const_name(transfer_name, mode));
 
 	if (is_region)
 		skip_poly_or_region(version);
@@ -1086,7 +1089,7 @@ int is_region;
 
 	if (verbose)
 		pm_message("%d x %d rectangle", pixwidth,
-			p.Bounds.bottom - p.Bounds.top, 0, 0, 0);
+			p.Bounds.bottom - p.Bounds.top);
 
 	colour_table = read_colour_table();
 
@@ -1103,7 +1106,7 @@ int is_region;
 	mode = read_word();
 
 	if (verbose)
-		pm_message("transfer mode = %s", const_name(transfer_name, mode),0,0,0,0);
+		pm_message("transfer mode = %s", const_name(transfer_name, mode));
 
 	if (is_region)
 		skip_poly_or_region(version);
@@ -1344,7 +1347,7 @@ int mode;
 	default:
 		if (mode != 0)
 			pm_message("no transfer function for code %s, using srcCopy",
-				const_name(transfer_name, mode), 0, 0, 0, 0);
+				const_name(transfer_name, mode));
 		return srcCopy;
 	}
 }
@@ -1424,7 +1427,7 @@ int mode;
 		dump_rect("to:          ", &clipdst);
 		pm_message("a %d x %d area to a %d x %d area",
 			rectwidth(&clipsrc), rectheight(&clipsrc),
-			rectwidth(&clipdst), rectheight(&clipdst), 0);
+			rectwidth(&clipdst), rectheight(&clipdst));
 	}
 
 	pkpixsize = 1;
@@ -1490,9 +1493,9 @@ int mode;
 		
 		pm_message("running 'pnmscale -xsize %d -ysize %d' on a %d x %d image",
 			rectwidth(&clipdst), rectheight(&clipdst),
-			rectwidth(&clipsrc), rectheight(&clipsrc), 0);
+			rectwidth(&clipsrc), rectheight(&clipsrc));
 
-		if (!(pnmscale = popen(command, "wb"))) {
+		if (!(pnmscale = popen(command, "w"))) {
 			pm_error("cannot execute command '%s'  popen() errno = %s (%d)",
                      command, strerror(errno), errno);
 		}
@@ -1813,11 +1816,11 @@ int pixelSize;
 				linelen = read_byte();
 
 			if (verbose > 1)
-				pm_message("linelen: %d", linelen, 0, 0, 0, 0);
+				pm_message("linelen: %d", linelen);
 
 			if (linelen > rowBytes) {
 				pm_message("linelen > rowbytes! (%d > %d) at line %d",
-					linelen, rowBytes, i, 0, 0);
+					linelen, rowBytes, i);
 			}
 
 			read_n(linelen, (char*) linebuf);
@@ -1948,10 +1951,10 @@ word* rowBytes;
 	p->pmReserved = read_long();
 
 	if (verbose) {
-		pm_message("pixelType: %d", p->pixelType, 0, 0, 0, 0);
-		pm_message("pixelSize: %d", p->pixelSize, 0, 0, 0, 0);
-		pm_message("cmpCount:  %d", p->cmpCount, 0, 0, 0, 0);
-		pm_message("cmpSize:   %d", p->cmpSize, 0, 0, 0, 0);
+		pm_message("pixelType: %d", p->pixelType);
+		pm_message("pixelSize: %d", p->pixelSize);
+		pm_message("cmpCount:  %d", p->cmpCount);
+		pm_message("cmpSize:   %d", p->cmpSize);
 	}
 
 	if (p->pixelType != 0)
@@ -1979,9 +1982,9 @@ read_colour_table()
 	ctSize = read_word();
 
 	if (verbose) {
-		pm_message("ctSeed:  %d", ctSeed, 0, 0, 0, 0);
-		pm_message("ctFlags: %d", ctFlags, 0, 0, 0, 0);
-		pm_message("ctSize:  %d", ctSize, 0, 0, 0, 0);
+		pm_message("ctSeed:  %ld", ctSeed);
+		pm_message("ctFlags: %d", ctFlags);
+		pm_message("ctSize:  %d", ctSize);
 	}
 
 	stage = "reading colour table";
@@ -2006,7 +2009,7 @@ read_colour_table()
 			pm_message("%d: [%d,%d,%d]", val,
 				colour_table[val].red,
 				colour_table[val].green,
-				colour_table[val].blue, 0);
+				colour_table[val].blue);
 	}
 
 	return colour_table;
@@ -2105,9 +2108,9 @@ struct Pattern* pat;
 	read_n(len = 8, (char*)buf);
 	if (verbose) {
 		pm_message("pattern: %02x%02x%02x%02x",
-			buf[0], buf[1], buf[2], buf[3], 0);
+			buf[0], buf[1], buf[2], buf[3]);
 		pm_message("pattern: %02x%02x%02x%02x",
-			buf[4], buf[5], buf[6], buf[7], 0);
+			buf[4], buf[5], buf[6], buf[7]);
 	}
 	exp = expand_buf(buf, &len, 1);
 	for (i = 0; i < 64; i++)
@@ -2120,7 +2123,7 @@ int version;
 	pen_height = read_word();
 	pen_width = read_word();
 	if (verbose)
-		pm_message("pen size %d x %d", pen_width, pen_height, 0, 0, 0);
+		pm_message("pen size %d x %d", pen_width, pen_height);
 }
 
 static void PnMode(version)
@@ -2132,7 +2135,7 @@ int version;
 		pen_mode -= 8;
 	if (verbose)
 		pm_message("pen transfer mode = %s",
-			const_name(transfer_name, pen_mode), 0, 0, 0, 0);
+			const_name(transfer_name, pen_mode));
 	
 	pen_trf = transfer(pen_mode);
 }
@@ -2151,7 +2154,7 @@ int v;
 	read_rgb(&foreground);
 	if (verbose)
 		pm_message("foreground now [%d,%d,%d]", 
-			foreground.red, foreground.green, foreground.blue, 0, 0);
+			foreground.red, foreground.green, foreground.blue);
 }
 
 static void RGBBkCol(v)
@@ -2160,7 +2163,7 @@ int v;
 	read_rgb(&background);
 	if (verbose)
 		pm_message("background now [%d,%d,%d]", 
-			background.red, background.green, background.blue, 0, 0);
+			background.red, background.green, background.blue);
 }
 
 static void read_point(p)
@@ -2312,7 +2315,7 @@ static void Line(v)
   read_point(&current);
   if (verbose)
     pm_message("(%d,%d) to (%d, %d)",
-	       p1.x,p1.y,current.x,current.y, 0);
+	       p1.x,p1.y,current.x,current.y);
   scan_line(p1.x,p1.y,current.x,current.y);
 }
 
@@ -2323,7 +2326,7 @@ static void LineFrom(v)
   read_point(&p1);
   if (verbose)
     pm_message("(%d,%d) to (%d, %d)",
-	       current.x,current.y,p1.x,p1.y, 0);
+	       current.x,current.y,p1.x,p1.y);
 
   if (!fullres)
 	  scan_line(current.x,current.y,p1.x,p1.y);
@@ -2340,7 +2343,7 @@ static void ShortLine(v)
   read_short_point(&current);
   if (verbose)
     pm_message("(%d,%d) delta (%d, %d)",
-	       p1.x,p1.y,current.x,current.y, 0);
+	       p1.x,p1.y,current.x,current.y);
   current.x += p1.x;
   current.y += p1.y;
 
@@ -2355,7 +2358,7 @@ static void ShortLineFrom(v)
   read_short_point(&p1);
   if (verbose)
     pm_message("(%d,%d) delta (%d, %d)",
-	       current.x,current.y,p1.x,p1.y, 0);
+               current.x,current.y,p1.x,p1.y);
   p1.x += current.x;
   p1.y += current.y;
   if (!fullres)
@@ -2589,7 +2592,7 @@ int version;
 	word frac = read_word();
 
 	if (verbose)
-		pm_message("PnLocHFrac = %d", frac, 0, 0, 0, 0);
+		pm_message("PnLocHFrac = %d", frac);
 }
 
 static void TxMode(version)
@@ -2601,7 +2604,7 @@ int version;
 		text_mode -= 8;
 	if (verbose)
 		pm_message("text transfer mode = %s",
-			const_name(transfer_name, text_mode), 0, 0, 0, 0);
+			const_name(transfer_name, text_mode));
 	
 	/* ignore the text mask bit 'cause we don't handle it yet */
 	text_trf = transfer(text_mode & ~64);
@@ -2612,7 +2615,7 @@ int version;
 {
 	text_font = read_word();
 	if (verbose)
-		pm_message("text font %s", const_name(font_name, text_font), 0, 0, 0, 0);
+		pm_message("text font %s", const_name(font_name, text_font));
 }
 
 static void TxFace(version)
@@ -2620,7 +2623,7 @@ int version;
 {
 	text_face = read_byte();
 	if (verbose)
-		pm_message("text face %d", text_face, 0, 0, 0, 0);
+		pm_message("text face %d", text_face);
 }
 
 static void TxSize(version)
@@ -2628,7 +2631,7 @@ int version;
 {
 	text_size = read_word();
 	if (verbose)
-		pm_message("text size %d", text_size, 0, 0, 0, 0);
+		pm_message("text size %d", text_size);
 }
 
 static void
@@ -2673,7 +2676,7 @@ int version;
 	dv = read_byte();
 
 	if (verbose)
-		pm_message("dh, dv = %d, %d", dh, dv, 0, 0, 0);
+		pm_message("dh, dv = %d, %d", dh, dv);
 
 	current.x += dh;
 	current.y += dv;
@@ -2856,7 +2859,7 @@ void picComment(word type, int length) {
 			pm_message("%s justification, %s flip, %d degree rotation, %d/2 linespacing",
 				const_name(ps_just_name, ps_just),
 				const_name(ps_flip_name, ps_flip),
-				ps_rotation, ps_linespace, 0);
+				ps_rotation, ps_linespace);
 		}
 		break;
 	case 151:
