@@ -936,11 +936,11 @@ int main(argc, argv)
 
     if (showLabel) {
 	PPM_ASSIGN(rgbcolour, Maxval, Maxval, Maxval);
-	sprintf(sysdesc, "System: %s\nPrimary illuminants (X, Y)\n\
+	sprintf(sysdesc, "System: %s\nPrimary illuminants (x, y)\n\
           Red:  %0.4f, %0.4f\n\
           Green: %0.4f, %0.4f\n\
           Blue:  %0.4f, %0.4f\n\
-White point (X, Y): %0.4f, %0.4f",
+White point (x, y): %0.4f, %0.4f", /* #176313 */
 	    cs->name, cs->xRed, cs->yRed, cs->xGreen, cs->yGreen,
 	    cs->xBlue, cs->yBlue, cs->xWhite, cs->yWhite);
 	ppmd_text(pixels, pixcols, pixrows, Maxval,
