@@ -1,5 +1,5 @@
 /*\
- * $Id: ppmtobmp.c,v 1.9 1992/11/24 19:39:33 dws Exp dws $
+ * $Id: ppmtobmp.c,v 1.2 2003/08/14 19:38:51 aba-guest Exp $
  *
  * ppmtobmp.c - Converts from a PPM file to a Microsoft Windows or OS/2
  * .BMP file.
@@ -17,6 +17,12 @@
  * without express or implied warranty.
  *
  * $Log: ppmtobmp.c,v $
+ * Revision 1.2  2003/08/14 19:38:51  aba-guest
+ * First part to 9.25 update; could now be really broken
+ *
+ * Revision 1.1.1.1  2003/08/12 18:23:03  aba-guest
+ * Latest debian release
+ *
  * Revision 1.9  1992/11/24  19:39:33  dws
  * Added copyright.
  *
@@ -433,8 +439,7 @@ BMPwritebits(FILE *fp, const unsigned long cx, const unsigned long cy,
 
                 if(rc == -1)
                 {
-                        pm_error("couldn't write row %d"
-                                 ,y);
+                        pm_error("couldn't write row %ld", y);
                 }
                 if(rc%4)
                 {
