@@ -60,6 +60,7 @@
 ** implied warranty.
 */
 
+#include <string.h>
 #include "ppm.h"
 #include "ppmcmap.h"
 #include "ppmfloyd.h"
@@ -117,7 +118,8 @@ static int runbyte1 ARGS((int bytes));
 static pixel * next_pixrow ARGS((FILE *fp, int row));
 static int * make_val_table ARGS((int oldmaxval, int newmaxval));
 static void * xmalloc ARGS((int bytes));
-#define MALLOC(n, type)     (type *)xmalloc((n) * sizeof(type))
+static void * xmalloc2 ARGS((int x, int y));
+#define MALLOC(n, type)     (type *)xmalloc2((n) , sizeof(type))
 static void init_read ARGS((FILE *fp, int *colsP, int *rowsP, pixval *maxvalP, int *formatP, int readall));
 static void write_body_rows ARGS((void));
 static void write_camg ARGS((void));
@@ -785,11 +787,16 @@ main(argc, argv)
 
     if( mode != MODE_CMAP ) {
         register int i;
+        overflow_add(cols, 15);
         coded_rowbuf = MALLOC(RowBytes(cols), unsigned char);
         for( i = 0; i < RowBytes(cols); i++ )
             coded_rowbuf[i] = 0;
         if( DO_COMPRESS )
+        {
+            overflow2(cols,2);
+            overflow_add(cols *2, 2);
             compr_rowbuf = MALLOC(WORSTCOMPR(RowBytes(cols)), unsigned char);
+        }
     }
 
     switch( mode ) {
@@ -981,7 +988,7 @@ compute_ham_cmap(cols, rows, maxval, maxcolors, colorsP, hbits)
     pm_message("initializing HAM colormap...");
 
     colors = 1<<(3*hbits);
-    hmap = malloc(colors * sizeof(hentry));
+    hmap = malloc2(colors,  sizeof(hentry));
     if (hmap == NULL)
         pm_error("Unable to allocate memory for HAM colormap.");
     hmaxval = pm_bitstomaxval(hbits);
@@ -1187,9 +1194,12 @@ ppm_to_ham(fp, cols, rows, maxval, colormap, colors, cmapmaxval, hamplanes)
         /*bodysize = do_ham_body(fp, NULL, cols, 
           rows, maxval, hammaxval, nPlanes, colbits, nocolor);*/
         if( bodysize > oldsize )
-            pm_message("warning - %s compression increases BODY size by %d%%", cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
+            pm_message("warning - %s compression increases BODY size "
+                       "by %ld%%", 
+                       cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
         else
-            pm_message("BODY compression (%s): %d%%", cmpNAME[compmethod], 100*(oldsize-bodysize)/oldsize);
+            pm_message("BODY compression (%s): %ld%%", 
+                       cmpNAME[compmethod], 100*(oldsize-bodysize)/oldsize);
 #if 0
         if( bodysize > oldsize && !compr_force ) {
             pm_message("%s compression would increase body size by %d%%", cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
@@ -1415,10 +1425,11 @@ ppm_to_deep(fp, cols, rows, maxval, bitspercolor)
     if( DO_COMPRESS ) {
         bodysize = do_deep_body(fp, NULL, cols, rows, maxval, bitspercolor);
         if( bodysize > oldsize )
-            pm_message("warning - %s compression increases BODY size by %d%%",
+            pm_message("warning - %s compression increases BODY size by %ld%%",
                        cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
         else
-            pm_message("BODY compression (%s): %d%%", cmpNAME[compmethod], 100*(oldsize-bodysize)/oldsize);
+            pm_message("BODY compression (%s): %ld%%", 
+                       cmpNAME[compmethod], 100*(oldsize-bodysize)/oldsize);
 #if 0
         if( bodysize > oldsize && !compr_force ) {
             pm_message("%s compression would increase body size by %d%%", cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
@@ -1549,10 +1560,12 @@ ppm_to_dcol(fp, cols, rows, maxval, dcol)
     if( DO_COMPRESS ) {
         bodysize = do_dcol_body(fp, NULL, cols, rows, maxval, dcol);
         if( bodysize > oldsize )
-            pm_message("warning - %s compression increases BODY size by %d%%",
-                       cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
+            pm_message("warning - %s compression increases BODY size by %ld%%",
+                       cmpNAME[compmethod], 
+                       100*(bodysize-oldsize)/oldsize);
         else
-            pm_message("BODY compression (%s): %d%%", cmpNAME[compmethod], 100*(oldsize-bodysize)/oldsize);
+            pm_message("BODY compression (%s): %ld%%", cmpNAME[compmethod], 
+                       100*(oldsize-bodysize)/oldsize);
 #if 0
         if( bodysize > oldsize && !compr_force ) {
             pm_message("%s compression would increase body size by %d%%", cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
@@ -1703,10 +1716,11 @@ ppm_to_std(fp, cols, rows, maxval, colormap, colors, cmapmaxval,
         bodysize = do_std_body(fp, NULL, cols, rows, maxval, colormap, 
                                colors, nPlanes);
         if( bodysize > oldsize )
-            pm_message("warning - %s compression increases BODY size by %d%%",
+            pm_message("warning - %s compression increases BODY size by %ld%%",
                        cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
         else
-            pm_message("BODY compression (%s): %d%%", cmpNAME[compmethod], 100*(oldsize-bodysize)/oldsize);
+            pm_message("BODY compression (%s): %ld%%", 
+                       cmpNAME[compmethod], 100*(oldsize-bodysize)/oldsize);
 #if 0
         if( bodysize > oldsize && !compr_force ) {
             pm_message("%s compression would increase body size by %d%%", cmpNAME[compmethod], 100*(bodysize-oldsize)/oldsize);
@@ -1832,6 +1846,7 @@ ppm_to_rgb8(ifp, cols, rows, maxval)
 
     maskmethod = 0;     /* no masking - RGB8 uses genlock bits */
     compmethod = 4;     /* RGB8 files are always compressed */
+    overflow2(cols, 4);
     compr_row = MALLOC(cols * 4, unsigned char);
 
     if( maxval != 255 ) {
@@ -1879,7 +1894,7 @@ ppm_to_rgb8(ifp, cols, rows, maxval)
         bodysize += compr_len;
     }
 
-    pm_message("BODY compression: %d%%", 100*(oldsize-bodysize)/oldsize);
+    pm_message("BODY compression: %ld%%", 100*(oldsize-bodysize)/oldsize);
 
     formsize =
         4 +                                 /* RGB8 */
@@ -1919,6 +1934,7 @@ ppm_to_rgbn(ifp, cols, rows, maxval)
 
     maskmethod = 0;     /* no masking - RGBN uses genlock bits */
     compmethod = 4;     /* RGBN files are always compressed */
+    overflow2(cols, 2);
     compr_row = MALLOC(cols * 2, unsigned char);
 
     if( maxval != 15 ) {
@@ -1978,7 +1994,7 @@ ppm_to_rgbn(ifp, cols, rows, maxval)
         bodysize += compr_len;
     }
 
-    pm_message("BODY compression: %d%%", 100*(oldsize-bodysize)/oldsize);
+    pm_message("BODY compression: %ld%%", 100*(oldsize-bodysize)/oldsize);
 
     formsize =
         4 +                                 /* RGBN */
@@ -2390,6 +2406,7 @@ make_val_table(oldmaxval, newmaxval)
     int i;
     int *table;
 
+    overflow_add(oldmaxval, 1);
     table = MALLOC(oldmaxval + 1, int);
     for(i = 0; i <= oldmaxval; i++ )
         table[i] = (i * newmaxval + oldmaxval/2) / oldmaxval;
@@ -2410,6 +2427,21 @@ xmalloc(bytes)
     return mem;
 }
 
+static void *
+xmalloc2(x,y)
+    int x;
+    int y;
+{
+    void *mem;
+
+    overflow2(x, y);
+
+    mem = malloc2(x, y);
+    if( mem == NULL )
+        pm_error("out of memory allocating %d bytes", x * y);
+    return mem;
+}
+
 
 static int  gFormat;
 static int  gCols;
