@@ -125,6 +125,8 @@ int main( argc, argv )
         medias = MSize_User;
 
         if (dpi300) {
+        	overflow2(medias.maxcols, 2);
+        	overflow2(medias.maxrows, 2);
                 medias.maxcols *= 2;
                 medias.maxrows *= 2;
         }
@@ -539,7 +541,7 @@ mymalloc(bytes)
 
     mem = malloc(bytes);
     if( mem == NULL )
-        pm_error("out of memory allocating %d bytes", bytes);
+        pm_error("out of memory allocating %ld bytes", bytes);
 
     return mem;
 }
