@@ -228,7 +228,7 @@ int end;                /* last frame to be extracted */
       exit(1);
     }  
 
-  inoffsets = (int *)malloc(no_frames*sizeof(int));
+  inoffsets = (int *)malloc2(no_frames, sizeof(int));
     
   if (fread (&(width),sizeof(int),1,inFile) != 1)
     {
@@ -653,8 +653,11 @@ ReadJPEG(mf, fp)
  * 00.03.30 Bryan changed "jpeglib.h" to <jpeglib.h> to make it work with
  * newer jpeg library.
  * 
- *  $Header: /u/smoot/md/mpeg_encode/RCS/jpeg.c,v 1.6 1995/06/08 20:36:00 smoot Exp $
+ *  $Header: /cvsroot/netpbm/netpbm/ppm/ppmtompeg/jpeg.c,v 1.1.1.1 2003/08/12 18:23:03 aba-guest Exp $
  *  $Log: jpeg.c,v $
+ *  Revision 1.1.1.1  2003/08/12 18:23:03  aba-guest
+ *  Latest debian release
+ *
  * Revision 1.6  1995/06/08  20:36:00  smoot
  * added "b"'s to fopen()s for MSDOS
  *
