@@ -20,6 +20,8 @@
 /*==============*
  * HEADER FILES *
  *==============*/
+#define _BSD_SOURCE 1
+    /* This makes sure that mkstemp() is in unistd.h */
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -343,7 +345,7 @@ int portNum;
 	  free(bigBuffer);
 	}
 
-	bigBuffer = (unsigned char *) malloc(bigBufferSize*
+	bigBuffer = (unsigned char *) malloc2(bigBufferSize,
 					     sizeof(unsigned char));
       }
 
@@ -557,7 +559,8 @@ int frameNumber;
   register int y;
   int	    numBytes;
   unsigned long data;
-  char    fileName[256];
+#define TMPFILE_TEMPLATE "/tmp/ppmtompeg.XXXXXX"    
+  char fileName[sizeof(TMPFILE_TEMPLATE)];
 
   Fsize_Note(frameNumber, yuvWidth, yuvHeight);
 
@@ -575,8 +578,12 @@ int frameNumber;
 
   if ( frameNumber != -1 ) {
     if ( separateConversion ) {
-      sprintf(fileName, "/tmp/foobar%d", machineNumber);
+      int fd;
+      strcpy(fileName, TMPFILE_TEMPLATE);
+      if (-1 == (fd = mkstemp(fileName)))
+          pm_error( "could not create temporary convolution file" );
       filePtr = fopen(fileName, "wb");
+      close(fd);
 
       /* read in stuff, SafeWrite to file, perform local conversion */
       do {
@@ -663,7 +670,7 @@ int portNum;
   
   TransmitPortNum(parallelHostName, portNum, combinePortNum);
   
-  frameDone = (boolean *) malloc(numInputFiles*sizeof(boolean));
+  frameDone = (boolean *) malloc2(numInputFiles, sizeof(boolean));
   memset((char *)frameDone, 0, numInputFiles*sizeof(boolean));
   
   if ( (ofp = fopen(outputFileName, "wb")) == NULL ) {
@@ -1269,6 +1276,8 @@ StartDecodeServer(numInputFiles, decodeFileName, parallelHostName, portNum)
 
     /* should keep list of port numbers to notify when frames become ready */
 
+    overflow2(numInputFiles, sizeof(int));
+    overflow2(numInputFiles, sizeof(boolean));
     ready = (boolean *) calloc(numInputFiles, sizeof(boolean));
     waitMachine = (int *) calloc(numInputFiles, sizeof(int));
     waitPort = (int *) malloc(numMachines*sizeof(int));
@@ -1950,8 +1959,17 @@ static int safe_fork(command)		/* fork child process and remember its PID */
  */
 
 /*  
- *  $Header: /n/picasso/project/mpeg/mpeg_dist/mpeg_encode/RCS/parallel.c,v 1.9 1995/08/16 18:22:08 smoot Exp $
+ *  $Header: /cvsroot/netpbm/netpbm/ppm/ppmtompeg/parallel.c,v 1.3 2003/11/11 20:16:57 aba-guest Exp $
  *  $Log: parallel.c,v $
+ *  Revision 1.3  2003/11/11 20:16:57  aba-guest
+ *  additional fix
+ *
+ *  Revision 1.2  2003/11/10 21:41:51  aba-guest
+ *  tmpfile-fixes
+ *
+ *  Revision 1.1.1.1  2003/08/12 18:23:03  aba-guest
+ *  Latest debian release
+ *
  *  Revision 1.9  1995/08/16 18:22:08  smoot
  *  indents
  *
