@@ -37,8 +37,11 @@
  */
 
 /*  
- *  $Header: /u/smoot/md/mpeg_encode/RCS/psearch.c,v 1.9 1995/01/19 23:09:12 eyhung Exp $
+ *  $Header: /cvsroot/netpbm/netpbm/ppm/ppmtompeg/psearch.c,v 1.1.1.1 2003/08/12 18:23:03 aba-guest Exp $
  *  $Log: psearch.c,v $
+ *  Revision 1.1.1.1  2003/08/12 18:23:03  aba-guest
+ *  Latest debian release
+ *
  * Revision 1.9  1995/01/19  23:09:12  eyhung
  * Changed copyrights
  *
@@ -299,6 +302,13 @@ SetSearchRange(pixelsP,pixelsB)
     ((searchRangeP>searchRangeB)?searchRangeP:searchRangeB)
       : ((searchRangeB>searchRangeB)?searchRangeB:searchRangeB);
     
+    overflow2(searchRangeP, 2);
+    overflow2(searchRangeB, 2);
+    overflow_add(searchRangeP*2, 3);
+    overflow_add(searchRangeB*2, 3);
+    overflow2(2*searchRangeB+3, sizeof(int));
+    overflow2(2*searchRangeP+3, sizeof(int));
+
     pmvHistogram = (int **) malloc((2*searchRangeP+3)*sizeof(int *));
     bbmvHistogram = (int **) malloc((2*searchRangeB+3)*sizeof(int *));
     bfmvHistogram = (int **) malloc((2*searchRangeB+3)*sizeof(int *));
@@ -891,6 +901,9 @@ ShowPMVHistogram(fpointer)
     int *columnTotals;
     int rowTotal;
 
+    overflow2(searchRangeP, 2);
+    overflow_add(searchRangeP*2,  3);
+    overflow2(searchRangeP*2+3, sizeof(int));
     columnTotals = (int *) calloc(2*searchRangeP+3, sizeof(int));
 
 #ifdef COMPLETE_DISPLAY
@@ -938,6 +951,9 @@ ShowBBMVHistogram(fpointer)
 
     fprintf(fpointer, "B-frame Backwards:\n");
 
+    overflow2(searchRangeB, 2);
+    overflow_add(searchRangeB*2,  3);
+    overflow2(searchRangeB*2+3, sizeof(int));
     columnTotals = (int *) calloc(2*searchRangeB+3, sizeof(int));
 
 #ifdef COMPLETE_DISPLAY
@@ -985,6 +1001,9 @@ ShowBFMVHistogram(fpointer)
 
     fprintf(fpointer, "B-frame Forwards:\n");
 
+    overflow2(searchRangeB, 2);
+    overflow_add(searchRangeB*2,  3);
+    overflow2(searchRangeB*2+3, sizeof(int));
     columnTotals = (int *) calloc(2*searchRangeB+3, sizeof(int));
 
 #ifdef COMPLETE_DISPLAY
