@@ -197,7 +197,7 @@ time(&diskStartTime);
       }
       *commandPtr = '\0';
 
-      if ( (ifp = popen(command, "rb")) == NULL ) {
+      if ( (ifp = popen(command, "r")) == NULL ) {
         fprintf(stderr, "ERROR:  Couldn't execute input conversion command:\n");
         fprintf(stderr, "\t%s\n", command);
         fprintf(stderr, "errno = %d\n", errno);
@@ -445,7 +445,7 @@ ReadIOConvert(fileName)
     }
     *commandPtr = '\0';
 
-    if ( (ifp = popen(command, "rb")) == NULL ) {
+    if ( (ifp = popen(command, "r")) == NULL ) {
 	fprintf(stderr, "ERROR:  Couldn't execute input conversion command:\n");
 	fprintf(stderr, "\t%s\n", command);
 	fprintf(stderr, "errno = %d\n", errno);
@@ -904,8 +904,14 @@ int w,h;
  */
 
 /*  
- *  $Header: /n/picasso/project/mpeg/mpeg_dist/mpeg_encode/RCS/readframe.c,v 1.27 1995/08/14 22:31:40 smoot Exp $
+ *  $Header: /cvsroot/netpbm/netpbm/ppm/ppmtompeg/readframe.c,v 1.2 2004/01/05 10:39:56 aba-guest Exp $
  *  $Log: readframe.c,v $
+ *  Revision 1.2  2004/01/05 10:39:56  aba-guest
+ *  changes for 10.0 / imported from sf.net
+ *
+ *  Revision 1.1.1.1  2003/08/12 18:23:03  aba-guest
+ *  Latest debian release
+ *
  *  Revision 1.27  1995/08/14 22:31:40  smoot
  *  reads training info from PPms now (needed for piping reads)
  *
