@@ -12,33 +12,33 @@
 
 /*
  * Changelog
- * 
+ *
+ * 2002/01/25 - Use pbm instead of ppm for alpha maps and fix alpha
+ *		sense. (black = transparent, white = opaque).
+ *		Option renamed to -andpbms; -andppms retained
+ *		for compatibility.
+ *		ppmtowinicon should now be the true reverse of winicontoppm.
+ *		Filled some "ignored" fields in ICO output for maximum
+ *		compatibility.  MVB.
+ *
  * 2000/05/24 - using colorhash_table instead of a straight array search
  *              Though it's a little slower for files due to the low # of
  *              colors, its neater.  LAB.
  */
 
+#include <string.h>
 #include <math.h>
 #include "winico.h"
 #include "ppm.h"
 #include "ppmcmap.h"
 
+extern void *realloc2(void *, int, int);
+
 #define MAJVERSION 0
 #define MINVERSION 3
 
 #define MAXCOLORS 256
 
-MS_Ico         createIconFile          ARGS((void));
-ICON_bmp       create1Bitmap           ARGS((pixel ** pa, int cols, int rows,colorhash_table cht));
-ICON_bmp       create4Bitmap           ARGS((pixel ** pa, int cols, int rows,colorhash_table cht));
-ICON_bmp       create8Bitmap           ARGS((pixel ** pa, int cols, int rows,colorhash_table cht));
-IC_InfoHeader  createInfoHeader        ARGS((IC_Entry entry, ICON_bmp xbmp, ICON_bmp abmp));
-void           addEntryToIcon          ARGS((MS_Ico MSIconData, char * xorPPM, char * andPPM));
-void           writeIC_Entry           ARGS((IC_Entry entry));
-void           writeIC_InfoHeader      ARGS((IC_InfoHeader ih));
-void           writeIC_Color           ARGS((IC_Color col));
-void           writeBitmap             ARGS((u1 ** bitmap, int xBytes, int height));
-void           writeMS_Ico             ARGS((MS_Ico MSIconData, char * outFname));
 
 static int      verbose = 0;
 static int      readAndMaps = 0;
@@ -50,9 +50,8 @@ static char *   outfname = "-";
 static FILE *   ofp;
 static FILE *   ifp;
 
-void PutByte(v) 
-int v;
-{
+static void 
+PutByte(int const v) {
    
    if (putc(v, ofp) == EOF)
      {
@@ -60,9 +59,8 @@ int v;
      }
 }
    
-void PutShort(v) 
-short v;
-{
+static void 
+PutShort(short const v) {
    
    if (pm_writelittleshort(ofp, v) == -1)
      {
@@ -71,9 +69,8 @@ short v;
    
 }
    
-void PutLong(v)
-long v;
-{
+static void 
+PutLong(long const v) {
    
    if (pm_writelittlelong(ofp, v) == -1)
      {
@@ -86,36 +83,32 @@ long v;
  * These have no purpose but to wrapper the Byte, Short & Long 
  * functions.
  */
-void writeU1 (v)
-u1 v;
-{
+static void 
+writeU1 (u1 const v) {
    file_offset++;
    PutByte(v);
 }
 
-void writeU1String (string, length) 
-char * string;
-int length;
-{
+static void 
+writeU1String (char * const string, int const length) {
    fwrite(string,sizeof(u1),length,ofp);
    file_offset += length;
 }
 
-void writeU2 (v) 
-u2 v;
-{
+static  void 
+writeU2 (u2 const v) {
    file_offset +=2;
    PutShort(v);
 }
 
-void writeU4 (v)
-u4 v;
-{
+static void 
+writeU4 (u4 const v) {
    file_offset += 4;
    PutLong(v);
 }
 
-MS_Ico createIconFile (void) {
+static MS_Ico 
+createIconFile (void) {
    MS_Ico MSIconData = malloc( sizeof (* MSIconData) );
    MSIconData->reserved     = 0;
    MSIconData->type         = 1;
@@ -124,23 +117,12 @@ MS_Ico createIconFile (void) {
    return MSIconData;
 }
 
-/*
- * Depending on if the image is stored as 1bpp, 4bpp or 8bpp, the 
- * encoding mechanism is different.
- * 
- * I didn't re-use the code from ppmtobmp since I need to keep the
- * bitmaps in memory till I've loaded all ppms.
- * 
- * 8bpp => 1 byte/palette index.
- * 4bpp => High Nibble, Low Nibble
- * 1bpp => 1 palette value per bit, high bit 1st.
- */
-ICON_bmp create1Bitmap (pa,cols,rows,cht)
-pixel ** pa;
-int cols;
-int rows;
-colorhash_table cht;
-{
+
+/* create andBitmap from pgm */
+
+static ICON_bmp 
+createAndBitmap (gray ** const ba, int const cols, int const rows,
+                 gray const maxval) {
    /*
     * How wide should the u1 string for each row be?
     * each byte is 8 pixels, but must be a multiple of 4 bytes.
@@ -154,12 +136,73 @@ colorhash_table cht;
       wt = (wt & ~3) + 4;
    }
    xBytes = wt;
-   rowData = malloc ( rows * sizeof (char *));
+   rowData = malloc2 ( rows , sizeof (char *));
+   icBitmap->xBytes = xBytes;
+   icBitmap->data   = rowData;
+   overflow2(xBytes, rows);
+   icBitmap->size   = xBytes * rows;
+   for (y=0;y<rows;y++) {
+      u1 * row = malloc2 ( xBytes, sizeof (u1));
+      int byteOn = 0;
+      int bitOn = 128;
+      memset (row, 0, xBytes);
+      rowData[rows-y-1] = row;
+      /* 
+       * Check there's a bit array, otherwise we're just faking this...
+       */
+      if (ba) {
+	 for (x=0;x<cols;x++) {
+            /* Black (bit clear) is transparent in PGM alpha maps,
+             * in ICO bit *set* is transparent.
+             */
+            if (ba[y][x] <= maxval/2) row[byteOn] |= bitOn;
+
+	    if (bitOn == 1) {
+	       byteOn++;
+	       bitOn = 128;
+	    } else {
+	       bitOn >>= 1;
+	    }
+	 }
+      }
+   }
+   return icBitmap;
+}
+
+
+/*
+ * Depending on if the image is stored as 1bpp, 4bpp or 8bpp, the 
+ * encoding mechanism is different.
+ * 
+ * I didn't re-use the code from ppmtobmp since I need to keep the
+ * bitmaps in memory till I've loaded all ppms.
+ * 
+ * 8bpp => 1 byte/palette index.
+ * 4bpp => High Nibble, Low Nibble
+ * 1bpp => 1 palette value per bit, high bit 1st.
+ */
+static ICON_bmp 
+create1Bitmap (pixel ** const pa, int const cols, int const rows, 
+               colorhash_table const cht) {
+   /*
+    * How wide should the u1 string for each row be?
+    * each byte is 8 pixels, but must be a multiple of 4 bytes.
+    */
+   ICON_bmp icBitmap = malloc ( sizeof (* icBitmap) );
+   int xBytes,y,x;
+   int wt = cols;
+   u1 ** rowData;
+   wt >>= 3;
+   if (wt & 3) {
+      wt = (wt & ~3) + 4;
+   }
+   xBytes = wt;
+   rowData = malloc2 ( rows,  sizeof (char *));
    icBitmap->xBytes = xBytes;
    icBitmap->data   = rowData;
    icBitmap->size   = xBytes * rows;
    for (y=0;y<rows;y++) {
-      u1 * row = malloc ( xBytes * sizeof (u1));
+      u1 * row = malloc2 ( xBytes , sizeof (u1));
       int byteOn = 0;
       int bitOn = 128;
       int value;
@@ -195,12 +238,9 @@ colorhash_table cht;
 }
 
 
-ICON_bmp create4Bitmap (pa,cols,rows,cht)
-pixel ** pa;
-int cols;
-int rows;
-colorhash_table cht;
-{
+static ICON_bmp 
+create4Bitmap (pixel ** const pa, int const cols, int const rows,
+               colorhash_table const cht) {
    /*
     * How wide should the u1 string for each row be?
     * each byte is 8 pixels, but must be a multiple of 4 bytes.
@@ -214,13 +254,15 @@ colorhash_table cht;
       wt = (wt & ~3) + 4;
    }
    xBytes = wt;
-   rowData = malloc ( rows * sizeof (char *));
+   rowData = malloc2 ( rows , sizeof (char *));
    icBitmap->xBytes = xBytes;
    icBitmap->data   = rowData;
+   overflow2(xBytes, rows);
    icBitmap->size   = xBytes * rows;
+   overflow2(xBytes, rows);
 
    for (y=0;y<rows;y++) {
-      u1 * row = malloc ( xBytes * sizeof (u1));
+      u1 * row = malloc2 ( xBytes, sizeof (u1));
       int byteOn = 0;
       int nibble = 1;   /* high nibble = 1, low nibble = 0; */
       int value;
@@ -253,12 +295,9 @@ colorhash_table cht;
 
 
 
-ICON_bmp create8Bitmap (pa,cols,rows,cht)
-pixel ** pa;
-int cols;
-int rows;
-colorhash_table cht;
-{
+static ICON_bmp 
+create8Bitmap (pixel ** const pa, int const cols, int const rows,
+               colorhash_table const cht) {
    /*
     * How wide should the u1 string for each row be?
     * each byte is 8 pixels, but must be a multiple of 4 bytes.
@@ -271,13 +310,14 @@ colorhash_table cht;
       wt = (wt & ~3) + 4;
    }
    xBytes = wt;
-   rowData = malloc ( rows * sizeof (char *));
+   rowData = malloc2 ( rows,  sizeof (char *));
    icBitmap->xBytes = xBytes;
    icBitmap->data   = rowData;
    icBitmap->size   = xBytes * rows;
+   overflow2(xBytes, rows);
 
    for (y=0;y<rows;y++) {
-      u1 * row = malloc ( xBytes * sizeof (u1));
+      u1 * row = malloc2 ( xBytes, sizeof (u1));
       memset (row, 0, xBytes);
       rowData[rows-y-1] = row;
       /* 
@@ -294,11 +334,9 @@ colorhash_table cht;
 
 
 
-IC_InfoHeader createInfoHeader(entry,xbmp,abmp)
-IC_Entry entry;
-ICON_bmp xbmp;
-ICON_bmp abmp;
-{
+static IC_InfoHeader 
+createInfoHeader(IC_Entry const entry, ICON_bmp const xbmp,
+                 ICON_bmp const abmp) {
    IC_InfoHeader ih  = malloc ( sizeof (* ih) );
    ih->size          = 40;
    ih->width         = entry->width;
@@ -306,31 +344,32 @@ ICON_bmp abmp;
    ih->planes        = 1;  
    ih->bitcount      = entry->bitcount;
    ih->compression   = 0;
-   ih->imagesize     = 0; /* uncompressed, so 0. */
+   ih->imagesize     = entry->width * entry->height * 8 / entry->bitcount;
    ih->x_pixels_per_m= 0;
    ih->y_pixels_per_m= 0;
-   ih->colors_used   = 0;
+   ih->colors_used   = 1 << entry->bitcount;
    ih->colors_important = 0;
    return ih;
 }
 
-IC_Palette createCleanPalette(void) {
+
+
+static IC_Palette 
+createCleanPalette(void) {
    IC_Palette palette = malloc ( sizeof (* palette) );
    int x;
-   palette->colors = malloc (MAXCOLORS * sizeof(IC_Color *));
+   palette->colors = malloc(MAXCOLORS * sizeof(IC_Color *));
    for (x=0;x<MAXCOLORS;x++ ){
       palette->colors[x] = NULL;
    }
    return palette;
 }
 
-void addColorToPalette(palette,i,r,g,b)
-IC_Palette palette;
-int i;
-int r;
-int g;
-int b;
-{
+
+
+static void 
+addColorToPalette(IC_Palette const palette, int const i,
+                  int const r, int const g, int const b) {
    palette->colors[i] = malloc ( sizeof (* palette->colors[i]) );
    palette->colors[i]->red      = r;
    palette->colors[i]->green    = g;
@@ -341,9 +380,9 @@ int b;
 
 
 static ICON_bmp 
-createBitmap (const int bpp, pixel ** const pa, 
-              const int cols, const int rows, 
-              const colorhash_table cht) {
+createBitmap (int const bpp, pixel ** const pa, 
+              int const cols, int const rows, 
+              colorhash_table const cht) {
     
     ICON_bmp retval;
     const int assumed_bpp = (pa == NULL) ? 1 : bpp;
@@ -365,15 +404,15 @@ createBitmap (const int bpp, pixel ** const pa,
 
 
 
-void addEntryToIcon (MSIconData,xorPPM,andPPM)
-MS_Ico MSIconData;
-char * xorPPM;
-char * andPPM;
-{
+static void 
+addEntryToIcon (MS_Ico const MSIconData, char * const xorPPM,
+                char * const andPGM) {
+
    IC_Entry entry = malloc ( sizeof (* entry) );
-   FILE * inPPM;
+   FILE * xorfile;
+   FILE * andfile;
    pixel ** xorPPMarray;
-   pixel ** andPPMarray;
+   gray ** andPGMarray;
    ICON_bmp xorBitmap;
    ICON_bmp andBitmap;
    int xorRows, xorCols, andRows, andCols;
@@ -384,41 +423,42 @@ char * andPPM;
    colorhash_table  xorCht;
    colorhash_table  andCht; 
    
-   pixval xorMaxP, andMaxP;
+   pixval xorMaxval;
+   gray andMaxval;
    /*
     * Read the xor PPM.
     */
-   inPPM = pm_openr(xorPPM);
-   xorPPMarray = ppm_readppm( inPPM, &xorCols, &xorRows, &xorMaxP );
-   pm_close(inPPM);
+   xorfile = pm_openr(xorPPM);
+   xorPPMarray = ppm_readppm( xorfile, &xorCols, &xorRows, &xorMaxval );
+   pm_close(xorfile);
    /*
     * Since the entry uses 1 byte to hold the width and height of the icon, the
     * image can't be more than 256 x 256.
     */
    if (xorRows > 255 || xorCols > 255) {
-      pm_error("Max size for a icon is 255 x 255 (1 byte fields)\n%s is %d x %d",xorPPM,xorCols,xorRows);
+      pm_error("Max size for a icon is 255 x 255 (1 byte fields)\n%s is "
+               "%d x %d", xorPPM, xorCols, xorRows);
    }
    
-   
+   if (verbose) pm_message("read PPM: maxval = %d", xorMaxval);
 
    /*
     * Figure out the colormap and turn it into the appropriate GIF
     * colormap - this code's pretty much straight from ppmtobpm
     */
    if (verbose) pm_message("computing colormap...");
-   xorChv = ppm_computecolorhist(xorPPMarray, xorCols, xorRows, MAXCOLORS, &colors);
+   xorChv = ppm_computecolorhist(xorPPMarray, xorCols, xorRows, MAXCOLORS, 
+                                 &colors);
    if (xorChv == (colorhist_vector) 0)
      pm_error("%s has too many colors - try doing a 'ppmquant %d'"
 	      , xorPPM, MAXCOLORS);
    if (verbose) pm_message("%d colors found", colors);
-   
-   if (verbose && (xorMaxP > 255))
-     {
-	pm_message("maxval is not 255 - automatically rescaling colors");
-     }
+
+   if (verbose && (xorMaxval > 255))
+       pm_message("maxval is not 255 - automatically rescaling colors");
    for (i = 0; i < colors; ++i)
      {
-	if (xorMaxP == 255)
+	if (xorMaxval == 255)
 	  {
 	     addColorToPalette(palette,i,
 			       PPM_GETR(xorChv[i].color),
@@ -428,9 +468,9 @@ char * andPPM;
 	else
 	  {
 	     addColorToPalette(palette,i,
-			       PPM_GETR(xorChv[i].color) * 255 / xorMaxP,
-			       PPM_GETG(xorChv[i].color) * 255 / xorMaxP,
-			       PPM_GETB(xorChv[i].color) * 255 / xorMaxP);
+			       PPM_GETR(xorChv[i].color) * 255 / xorMaxval,
+			       PPM_GETG(xorChv[i].color) * 255 / xorMaxval,
+			       PPM_GETB(xorChv[i].color) * 255 / xorMaxval);
 	  }
      }
    
@@ -455,49 +495,27 @@ char * andPPM;
       bpp = 8;
       entry_cols = 256;
    }
-   if (!strlen (andPPM)) {
-      /*
-       * They're not supplying a bitmap for 'and'.
-       * Fake the bitmap.
-       */
-      andPPMarray = NULL;
+   if (!strlen (andPGM)) {
+      /* He's not supplying a bitmap for 'and'.  Fake the bitmap. */
+      andPGMarray = NULL;
       andCols = xorCols;
       andRows = xorRows;
-      andMaxP = 1;
+      andMaxval = 1;
       andCht  = NULL;
 
    } else {
-      /* to check for 2 colors max */
-      colorhist_vector tempchv;
-      int colors = 0;
-      
-      inPPM = pm_openr(andPPM);
-      andPPMarray = ppm_readppm( inPPM, &andCols, &andRows, &andMaxP );
-      pm_close(inPPM);
-      /*
-       * We check that there are only 2 palette entries here.
-       * 
-       * This could be fixed by using a pbm, but then I'd have to write them 
-       * in winicontoppm, which wouldnt be toppm anymore, etc...
-       * 
-       * Since we're only looking at black and not black, there's no need to 
-       * scale.
-       */
-      tempchv = ppm_computecolorhist(andPPMarray, xorCols, xorRows, 2, &colors);
-      if (tempchv == (colorhist_vector) 0)
-	     pm_error("%s should have only 2 colors", andPPM);
-      andCht = ppm_colorhisttocolorhash(tempchv, colors);
-      ppm_freecolorhist(tempchv);
-      /*
-       * Need to check and & xor are same height & size.
-       */
+      andfile = pm_openr(andPGM);
+      andPGMarray = pgm_readpgm( andfile, &andCols, &andRows, &andMaxval );
+      pm_close(andfile);
+
       if ((andCols != xorCols) || (andRows != xorRows)) {
-	 pm_error("%s and %s have different dimensions.\nAborting.\n",xorPPM,andPPM);
+          pm_error("%s and %s have different dimensions.\nAborting.\n",
+                   xorPPM, andPGM);
       }
    }
    if (verbose) pm_message ("Dimensions of ppms %d, %d\n",xorCols,xorRows);
    xorBitmap = createBitmap(bpp,xorPPMarray,xorCols,xorRows,xorCht);
-   andBitmap = createBitmap(1,  andPPMarray,xorCols,xorRows,andCht);
+   andBitmap = createAndBitmap(andPGMarray, andCols, andRows, andMaxval);
    /*
     * Fill in the entry data fields.
     */
@@ -512,8 +530,16 @@ char * andPPM;
    entry->bitcount      = bpp;
    entry->ih            = createInfoHeader(entry,xorBitmap,andBitmap);
    entry->colors        = palette->colors;
-   entry->size_in_bytes = xorBitmap->size + andBitmap->size + 40 + (4 * entry->color_count);
-   if (verbose) pm_message ("entry->size_in_bytes = %d + %d + %d = %d\n",xorBitmap->size ,andBitmap->size , 40,entry->size_in_bytes );
+   overflow2(4, entry->color_count);
+   overflow_add(xorBitmap->size, andBitmap->size);
+   overflow_add(xorBitmap->size + andBitmap->size, 40);
+   overflow_add(xorBitmap->size + andBitmap->size + 40, 4 * entry->color_count);
+   entry->size_in_bytes = 
+       xorBitmap->size + andBitmap->size + 40 + (4 * entry->color_count);
+   if (verbose) 
+       pm_message ("entry->size_in_bytes = %d + %d + %d = %d\n",
+                   xorBitmap->size ,andBitmap->size, 
+                   40, entry->size_in_bytes );
    /*
     * We don't know the offset ATM, set to 0 for now.
     * Have to calculate this at the end.
@@ -526,17 +552,18 @@ char * andPPM;
    /*
     * Add the entry to the entries array.
     */
+   overflow_add(MSIconData->count,1);
    MSIconData->count++;
    /* 
     * Perhaps I should use something that allocs a decent amount at start...
     */
-   MSIconData->entries = realloc (MSIconData->entries, MSIconData->count * sizeof(IC_Entry *));   
+   MSIconData->entries = 
+       realloc2(MSIconData->entries, MSIconData->count, sizeof(IC_Entry *));
    MSIconData->entries[MSIconData->count-1] = entry;
 }
 
-void writeIC_Entry (entry)
-IC_Entry entry;
-{
+static void 
+writeIC_Entry (IC_Entry const entry) {
    writeU1(entry->width);
    writeU1(entry->height);
    writeU1(entry->color_count); /* chops 256->0 on its own.. */
@@ -547,30 +574,32 @@ IC_Entry entry;
    writeU4(entry->file_offset);
 }
 
-void writeIC_InfoHeader (ih)
-IC_InfoHeader ih;
-{
+
+
+static void 
+writeIC_InfoHeader (IC_InfoHeader const ih) {
    writeU4(ih->size);
    writeU4(ih->width);
    writeU4(ih->height);
    writeU2(ih->planes);
    writeU2(ih->bitcount);
-   writeU4(ih->imagesize);
    writeU4(ih->compression);
+   writeU4(ih->imagesize);
    writeU4(ih->x_pixels_per_m);
    writeU4(ih->y_pixels_per_m);
    writeU4(ih->colors_used);
    writeU4(ih->colors_important);
 }
 
-void writeIC_Color (col)
-IC_Color col;
-{
-   /*
-    * Since the ppm might not have as many colors in it as we'd like, (2, 16, 256),
-    * stick 0 in the gaps.
+
+
+static void 
+writeIC_Color (IC_Color const col) {
+   /* Since the ppm might not have as many colors in it as we'd like,
+    * (2, 16, 256), stick 0 in the gaps.
     * 
-    * This means that we lose palette information, but that can't be helped.
+    * This means that we lose palette information, but that can't be
+    * helped.  
     */
    if (col == NULL) {
       writeU1(0);
@@ -586,11 +615,9 @@ IC_Color col;
 }
 
 
-void writeBitmap(bitmap,xBytes,height)
-u1 ** bitmap;
-int xBytes;
-int height;
-{
+
+static void
+writeBitmap(u1 ** const bitmap, int const xBytes, int const height) {
    int y;
    for (y = 0;y<height;y++) {
       fwrite (bitmap[y],1,xBytes,ofp);
@@ -598,10 +625,10 @@ int height;
    }
 }
 
-void writeMS_Ico (MSIconData,outFname)
-MS_Ico MSIconData;
-char * outFname;
-{
+
+
+static void 
+writeMS_Ico (MS_Ico const MSIconData, char * const outFname) {
    int x,y;
    if (!strcmp(outfname,"-")) {
       ofp = stdout;
@@ -619,17 +646,21 @@ char * outFname;
 	 writeIC_Color(MSIconData->entries[x]->colors[y]);
       }
       if (verbose) pm_message("writing xor bitmap\n");
-      writeBitmap(MSIconData->entries[x]->xorBitmapOut,MSIconData->entries[x]->xBytesXor,MSIconData->entries[x]->height);
+      writeBitmap(MSIconData->entries[x]->xorBitmapOut,
+                  MSIconData->entries[x]->xBytesXor,
+                  MSIconData->entries[x]->height);
       if (verbose) pm_message("writing and bitmap\n");
-      writeBitmap(MSIconData->entries[x]->andBitmapOut,MSIconData->entries[x]->xBytesAnd,MSIconData->entries[x]->height);
+      writeBitmap(MSIconData->entries[x]->andBitmapOut,
+                  MSIconData->entries[x]->xBytesAnd,
+                  MSIconData->entries[x]->height);
    }
    fclose (ofp);
 }
 
-int main (argc,argv)
-int argc;
-char ** argv;
-{
+
+
+int 
+main (int argc, char ** argv) {
    MS_Ico MSIconData = createIconFile();
    int iconOn = 1;
    int argn;
@@ -651,7 +682,8 @@ char ** argv;
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
       if (pm_keymatch(argv[argn], "-verbose", 2))
 	verbose++;
-      else if (pm_keymatch(argv[argn], "-andppms", 2)) {
+      else if (pm_keymatch(argv[argn], "-andpgms", 2) 
+               || pm_keymatch(argv[argn], "-andppms", 2)) {
 	readAndMaps++;
       }
       else if (pm_keymatch(argv[argn], "-output", 2)) {
