@@ -162,11 +162,11 @@ typedef struct {
 /* This routine reads a rgb text file.  It stores the rgb values (0->65535)
    and the rgb mnemonics (malloc'ed) into the "rgbn" array.  Returns the
    number of entries stored in "rgbn_max". */
-void
+static void
 read_rgb_names(rgb_fname, rgbn, rgbn_max)
     char *rgb_fname;
     rgb_names rgbn[MAX_RGBNAMES];
-int *rgbn_max;
+    int *rgbn_max;
 
 {
     FILE *rgbf;
@@ -175,7 +175,7 @@ int *rgbn_max;
 
     /* Open the rgb text file.  Abort if error. */
     if ((rgbf = fopen(rgb_fname, "r")) == NULL)
-    pm_error("error opening rgb text file \"%s\"", rgb_fname, 0, 0, 0, 0);
+    pm_error("error opening rgb text file \"%s\"", rgb_fname);
 
     /* Loop reading each line in the file. */
     for (i = 0; fgets(line, sizeof(line), rgbf); i++) {
@@ -207,8 +207,9 @@ int *rgbn_max;
         continue;
     }
     /* Allocate memory for ascii name.  Abort if error. */
+    overflow_add(strlen(name), 1);
     if (!(rgbname = (char *) malloc(strlen(name) + 1)))
-        pm_error("out of memory allocating rgb name", 0, 0, 0, 0, 0);
+        pm_error("out of memory allocating rgb name");
         
 #ifdef NAMESLOWCASE
     /* Copy string to ascii name and lowercase it. */
@@ -235,7 +236,7 @@ int *rgbn_max;
 
 
 
-char *
+static char *
 gen_numstr(int const input, int const digits) {
 /*---------------------------------------------------------------------------
    Given a number and a base (MAXPRINTABLE), this routine prints the
@@ -251,6 +252,7 @@ gen_numstr(int const input, int const digits) {
     int i;
 
     /* Allocate memory for printed number.  Abort if error. */
+    overflow_add(digits, 1);
     if (!(str = (char *) malloc(digits + 1)))
         pm_error("out of memory");
 
@@ -273,7 +275,7 @@ gen_numstr(int const input, int const digits) {
 
 
 
-void
+static void
 gen_cmap(colorhist_vector const chv, int const ncolors, 
          pixval const maxval, int map_rgb_names, 
          rgb_names rgbn[MAX_RGBNAMES], int const rgbn_max,
@@ -292,7 +294,8 @@ gen_cmap(colorhist_vector const chv, int const ncolors,
     char *str;
     cixel_map * cmap;  /* malloc'ed */
 
-    cmap = malloc(sizeof(cixel_map) * (ncolors+1));
+    overflow_add(ncolors,1);
+    cmap = malloc2(sizeof(cixel_map), (ncolors+1));
     if (cmapP == NULL)
         pm_error("Out of memory allocating %d bytes for a color map.",
                  sizeof(cixel_map) * (ncolors+1));
@@ -384,8 +387,9 @@ gen_cmap(colorhist_vector const chv, int const ncolors,
         if (r == rgbn[j].r && g == rgbn[j].g && b == rgbn[j].b) {
 
             /* Matched.  Allocate string, copy mnemonic, and exit. */
+            overflow_add(strlen(rgbn[j].name), 1);
             if (!(str = (char *) malloc(strlen(rgbn[j].name) + 1)))
-            pm_error("out of memory", 0, 0, 0, 0, 0);
+            pm_error("out of memory");
             strcpy(str, rgbn[j].name);
             cmap[i].rgbname = str;
             matched = 1;
@@ -403,7 +407,7 @@ gen_cmap(colorhist_vector const chv, int const ncolors,
                     mval == 0x00FF ? 8 :
                     mval == 0x0FFF ? 11 :
                     14)))
-        pm_error("out of memory", 0, 0, 0, 0, 0);
+        pm_error("out of memory");
 
     sprintf(str, mval == 0x000F ? "#%X%X%X" :
         mval == 0x00FF ? "#%02X%02X%02X" :
@@ -505,7 +509,7 @@ read_alpha(char filespec[], gray *** const alphaP,
     
     if (cols != alphacols || rows != alpharows)
         pm_error("Alpha mask is not the same dimensions as the "
-                 "image.  Image is %d by %d, while mask is %d x %s.",
+                 "image.  Image is %d by %d, while mask is %d x %d.",
                  cols, rows, alphacols, alpharows);
 }
     
