@@ -27,7 +27,7 @@
  * Date:	1 April 1981
  * Copyright (c) 1981,1986 Spencer W. Thomas
  *
- * $Id: rle_putrow.c,v 3.0.1.2 1992/01/28 18:29:22 spencer Exp $
+ * $Id: rle_putrow.c,v 1.2 2003/08/14 19:38:51 aba-guest Exp $
  */
  
 #include "stdio.h"
@@ -458,10 +458,8 @@ register rle_hdr * the_hdr;
     Setup();
 }
 
-/*ARGSUSED*/
 void
-DefaultBlockHook(the_hdr)
-rle_hdr * the_hdr;
+DefaultBlockHook(rle_hdr * the_hdr)
 {
     					/* Do nothing */
 }
